@@ -35,62 +35,54 @@
  *
  */
 
-#ifndef PROPERTIES_MODEL_H_
-#define PROPERTIES_MODEL_H_
+#pragma once
 
-#include <pcl/apps/cloud_composer/qt.h>
-#include <pcl/common/boost.h> 
+#include <QStandardItemModel>
 
+namespace pcl {
+namespace cloud_composer {
+class CloudComposerItem;
+class PropertiesModel : public QStandardItemModel {
+  Q_OBJECT
+public:
+  /** \brief Constructor used for tool parameters */
+  PropertiesModel(QObject* parent = nullptr);
+  /** \brief Constructor used for item parameters */
+  PropertiesModel(CloudComposerItem* parent_item, QObject* parent = nullptr);
+  PropertiesModel(const PropertiesModel& to_copy);
 
-namespace pcl
-{
-  namespace cloud_composer
-  {
-    class CloudComposerItem;
-    class PropertiesModel : public QStandardItemModel
-    {
-      Q_OBJECT
-      public:
-        
-        /** \brief Constructor used for tool parameters */
-        PropertiesModel (QObject *parent = 0);
-        /** \brief Constructor used for item parameters */
-        PropertiesModel (CloudComposerItem* parent_item, QObject *parent = 0);
-        PropertiesModel (const PropertiesModel& to_copy);
-        virtual ~PropertiesModel ();
-        
-        /** \brief Helper function for adding a new property */
-        void
-        addProperty (const QString prop_name, const QVariant value, const Qt::ItemFlags flags = Qt::ItemIsSelectable, const QString category = "");
-        
-        /** \brief Helper function for adding a new property category */
-        void
-        addCategory (const QString category_name);
-        
-        /** \brief Helper function to get a property */
-        QVariant 
-        getProperty (const QString prop_name) const;
-        
-        void 
-        copyProperties (const PropertiesModel* to_copy);
-        
-      public Q_SLOTS:
-        void
-        propertyChanged (QStandardItem* property_item);
-      
-      Q_SIGNALS:
-        void 
-        propertyChanged (const QStandardItem* property_item, const CloudComposerItem* parent_item_);
-        
-        
-      private:
-        CloudComposerItem* parent_item_;
-        
-     };
-  }
-}
+  /** \brief Helper function for adding a new property */
+  void
+  addProperty(const QString& prop_name,
+              const QVariant& value,
+              const Qt::ItemFlags flags = Qt::ItemIsSelectable,
+              const QString& category = "");
 
-Q_DECLARE_METATYPE (pcl::cloud_composer::PropertiesModel);
-Q_DECLARE_METATYPE (pcl::cloud_composer::PropertiesModel*);
+  /** \brief Helper function for adding a new property category */
+  void
+  addCategory(const QString& category_name);
 
-#endif //PROPERTIES_MODEL_H_
+  /** \brief Helper function to get a property */
+  QVariant
+  getProperty(const QString& prop_name) const;
+
+  void
+  copyProperties(const PropertiesModel* to_copy);
+
+public Q_SLOTS:
+  void
+  propertyChanged(QStandardItem* property_item);
+
+Q_SIGNALS:
+  void
+  propertyChanged(const QStandardItem* property_item,
+                  const CloudComposerItem* parent_item_);
+
+private:
+  CloudComposerItem* parent_item_;
+};
+} // namespace cloud_composer
+} // namespace pcl
+
+Q_DECLARE_METATYPE(pcl::cloud_composer::PropertiesModel);
+Q_DECLARE_METATYPE(pcl::cloud_composer::PropertiesModel*);
