@@ -35,79 +35,73 @@
  *
  */
 
-#ifndef NORMAL_ESTIMATION_H_
-#define NORMAL_ESTIMATION_H_
+#pragma once
 
 #include <pcl/apps/cloud_composer/tool_interface/abstract_tool.h>
 #include <pcl/apps/cloud_composer/tool_interface/tool_factory.h>
 
+namespace pcl {
+namespace cloud_composer {
+class NormalEstimationTool : public NewItemTool {
+  Q_OBJECT
+public:
+  NormalEstimationTool(PropertiesModel* parameter_model, QObject* parent);
 
-namespace pcl
-{
-  namespace cloud_composer
+  QList<CloudComposerItem*>
+  performAction(QList<const CloudComposerItem*> input_data,
+                PointTypeFlags::PointType type = PointTypeFlags::NONE) override;
+
+  inline QString
+  getToolName() const override
   {
-    class NormalEstimationTool : public NewItemTool
-    {
-      Q_OBJECT
-      public:
-        NormalEstimationTool (PropertiesModel* parameter_model, QObject* parent);
-        virtual ~NormalEstimationTool ();
-        
-       virtual QList <CloudComposerItem*>
-        performAction (QList <const CloudComposerItem*> input_data, PointTypeFlags::PointType type = PointTypeFlags::NONE);
-      
-        inline virtual QString
-        getToolName () const { return "Normal Estimation Tool";}
-    };
-
-    
-    class NormalEstimationToolFactory : public QObject, public ToolFactory
-    {
-      Q_OBJECT
-      Q_INTERFACES (pcl::cloud_composer::ToolFactory)
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-      Q_PLUGIN_METADATA(IID "cloud_composer.ToolFactory/1.0")
-#endif
-      public:
-        NewItemTool*
-        createTool (PropertiesModel* parameter_model, QObject* parent = 0) 
-        {
-            return new NormalEstimationTool(parameter_model, parent);
-        }
-        
-        PropertiesModel*
-        createToolParameterModel (QObject* parent);
-        
-        inline virtual QString 
-        getPluginName () const { return "Normal Estimation";}
-        
-        virtual QString 
-        getToolGroupName () const { return "Feature Estimation";}
-        
-        virtual QString
-        getIconName () const { return ":/normal_estimation.png"; }
-        
-        inline virtual CloudComposerItem::ItemType
-        getInputItemType () const
-        {
-          return CloudComposerItem::CLOUD_ITEM;
-        }
-        
-        inline virtual QList <CloudComposerItem::ItemType>
-        getRequiredInputChildrenTypes () const 
-        {
-          return QList <CloudComposerItem::ItemType> ();
-        }
-    };
-
-
-
+    return "Normal Estimation Tool";
   }
-}
+};
 
+class NormalEstimationToolFactory : public QObject, public ToolFactory {
+  Q_OBJECT
+  Q_INTERFACES(pcl::cloud_composer::ToolFactory)
+  Q_PLUGIN_METADATA(IID "cloud_composer.ToolFactory/1.0")
+public:
+  NewItemTool*
+  createTool(PropertiesModel* parameter_model, QObject* parent = nullptr) override
+  {
+    return new NormalEstimationTool(parameter_model, parent);
+  }
 
+  PropertiesModel*
+  createToolParameterModel(QObject* parent) override;
 
+  inline QString
+  getPluginName() const override
+  {
+    return "Normal Estimation";
+  }
 
+  QString
+  getToolGroupName() const override
+  {
+    return "Feature Estimation";
+  }
 
+  QString
+  getIconName() const override
+  {
+    return ":/normal_estimation.png";
+  }
 
-#endif //NORMAL_ESTIMATION_H_
+  inline CloudComposerItem::ItemType
+  getInputItemType() const override
+  {
+    return CloudComposerItem::CLOUD_ITEM;
+  }
+
+  inline QList<CloudComposerItem::ItemType>
+  getRequiredInputChildrenTypes() const override
+  {
+    return QList<CloudComposerItem::ItemType>();
+  }
+};
+
+} // namespace cloud_composer
+} // namespace pcl
