@@ -2,12 +2,12 @@
 
 /////////// MAIN ////////////////////
 int
-main (int argc, char ** argv)
+main(int argc, char** argv)
 {
   // Initialize QT
-  QApplication app (argc, argv);
-  
+  QApplication app(argc, argv);
+
   pcl::cloud_composer::ComposerMainWindow cc;
-  cc.show ();
-  return (app.exec ());
-}
\ No newline at end of file
+  cc.show();
+  return (QApplication::exec());
+}
