@@ -40,21 +40,9 @@
 #include <pcl/pcl_config.h>
 #include <pcl/io/dinast_grabber.h>
 
-using namespace std;
-
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 pcl::DinastGrabber::DinastGrabber (const int device_position)
-  : image_width_ (320)
-  , image_height_ (240)
-  , sync_packet_size_ (512)
-  , fov_ (64. * M_PI / 180.)
-  , context_ (NULL)
-  , bulk_ep_ (std::numeric_limits<unsigned char>::max ())
-  , second_image_ (false)
-  , running_ (false)
 {
-  image_size_ = image_width_ * image_height_;
-  dist_max_2d_ = 1. / (image_width_ / 2.);
   onInit(device_position);
   
   point_cloud_signal_ = createSignal<sig_cb_dinast_point_cloud> ();
@@ -62,7 +50,7 @@ pcl::DinastGrabber::DinastGrabber (const int device_position)
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-pcl::DinastGrabber::~DinastGrabber () throw ()
+pcl::DinastGrabber::~DinastGrabber () noexcept
 {
   try
   {
@@ -96,7 +84,7 @@ pcl::DinastGrabber::getFramesPerSecond () const
 { 
   static double last = pcl::getTime ();
   double now = pcl::getTime (); 
-  float rate = 1 / float(now - last);
+  float rate = 1 / static_cast<float>(now - last);
   last = now; 
 
   return (rate);
@@ -107,7 +95,7 @@ void
 pcl::DinastGrabber::onInit (const int device_position)
 {
   setupDevice (device_position);
-  capture_thread_ = boost::thread (&DinastGrabber::captureThreadFunction, this);
+  capture_thread_ = std::thread (&DinastGrabber::captureThreadFunction, this);
   image_ = new unsigned char [image_size_];
 }
 
@@ -125,9 +113,13 @@ pcl::DinastGrabber::setupDevice (int device_position, const int id_vendor, const
     sstream << "[pcl::DinastGrabber::setupDevice] libusb initialization failure, LIBUSB_ERROR: "<< ret;
     PCL_THROW_EXCEPTION (pcl::IOException, sstream.str ());
   }
-  
-  libusb_set_debug (context_, 3);
-  libusb_device **devs = NULL;
+
+  #if defined(LIBUSB_API_VERSION) && (LIBUSB_API_VERSION >= 0x01000106)
+    libusb_set_option (context_, LIBUSB_OPTION_LOG_LEVEL, 3);
+  #else
+    libusb_set_debug (context_, 3);
+  #endif
+  libusb_device **devs = nullptr;
   
   // Get the list of USB devices
   ssize_t number_devices = libusb_get_device_list (context_, &devs);
@@ -155,13 +147,13 @@ pcl::DinastGrabber::setupDevice (int device_position, const int id_vendor, const
     libusb_get_config_descriptor (devs[i], 0, &config);
 
     // Iterate over all interfaces available
-    for (int f = 0; f < int (config->bNumInterfaces); ++f)
+    for (int f = 0; f < static_cast<int>(config->bNumInterfaces); ++f)
     {
       // Iterate over the number of alternate settings
       for (int j = 0; j < config->interface[f].num_altsetting; ++j)
       {
         // Iterate over the number of end points present
-        for (int k = 0; k < int (config->interface[f].altsetting[j].bNumEndpoints); ++k) 
+        for (int k = 0; k < static_cast<int>(config->interface[f].altsetting[j].bNumEndpoints); ++k) 
         {
           if (config->interface[f].altsetting[j].endpoint[k].bmAttributes == LIBUSB_TRANSFER_TYPE_BULK)
           {
@@ -183,7 +175,7 @@ pcl::DinastGrabber::setupDevice (int device_position, const int id_vendor, const
   libusb_free_device_list (devs, 1);
   
   // Check if device founded if not notify
-  if (device_handle_ == NULL)
+  if (device_handle_ == nullptr)
     PCL_THROW_EXCEPTION (pcl::IOException, "[pcl::DinastGrabber::setupDevice] Failed to find any DINAST devices attached");
   
   // Claim device interface
@@ -201,7 +193,7 @@ pcl::DinastGrabber::getDeviceVersion ()
      PCL_THROW_EXCEPTION (pcl::IOException, "[pcl::DinastGrabber::getDeviceVersion] Error trying to get device version");
  
   //data[21] = 0;
-  return (std::string (reinterpret_cast<const char*> (data)));
+  return {reinterpret_cast<const char*>(data)};
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -251,7 +243,7 @@ pcl::DinastGrabber::readImage ()
                                     RGB16 * (image_size_) + sync_packet_size_, &actual_length, 1000);
     if (res != 0 || actual_length == 0)
     {
-      memset (&image_[0], 0x00, image_size_);
+      std::fill_n(image_, image_size_, 0x00);
       PCL_THROW_EXCEPTION (pcl::IOException, "[pcl::DinastGrabber::readImage] USB read error!");
     }
 
@@ -259,7 +251,7 @@ pcl::DinastGrabber::readImage ()
     PCL_DEBUG ("[pcl::DinastGrabber::readImage] Read: %d, size of the buffer: %d\n" ,actual_length, g_buffer_.size ());
     
     // Copy data into the buffer
-    int back = int (g_buffer_.size ());
+    int back = static_cast<int>(g_buffer_.size ());
     g_buffer_.resize (back + actual_length);
 
     for (int i = 0; i < actual_length; ++i)
@@ -313,7 +305,7 @@ pcl::DinastGrabber::getXYZIPointCloud ()
     {
       double pixel = image_[x + image_width_ * y];
 
-      // Correcting distortion, data emprically got in a calibration test
+      // Correcting distortion, data empirically got in a calibration test
       double xc = static_cast<double> (x - image_width_ / 2);
       double yc = static_cast<double> (y - image_height_ / 2);
       double r1 = sqrt (xc * xc + yc * yc);
@@ -326,28 +318,28 @@ pcl::DinastGrabber::getXYZIPointCloud ()
       if (pixel > A)
         pixel = A;
       double dy = y*0.1;
-      double dist = (log (static_cast<double> (pixel / A)) / B - dy) * (7E-07*r3 - 0.0001*r2 + 0.004*r1 + 0.9985) * 1.5;
+      double dist = (std::log (static_cast<double> (pixel / A)) / B - dy) * (7E-07*r3 - 0.0001*r2 + 0.004*r1 + 0.9985) * 1.5;
       double theta_colati = fov_ * r1 * dist_max_2d_;
-      double c_theta = cos (theta_colati);
+      double c_theta = std::cos (theta_colati);
       double s_theta = sin (theta_colati);
       double c_ksai = static_cast<double> (x - 160) / r1;
       double s_ksai = static_cast<double> (y - 120) / r1;
-      cloud->points[depth_idx].x = static_cast<float> ((dist * s_theta * c_ksai) / 500.0 + 0.5);
-      cloud->points[depth_idx].y = static_cast<float> ((dist * s_theta * s_ksai) / 500.0 + 0.5);
-      cloud->points[depth_idx].z = static_cast<float> (dist * c_theta);
-      if (cloud->points[depth_idx].z < 0.01f)
-        cloud->points[depth_idx].z = 0.01f;
-      cloud->points[depth_idx].z /= 500.0f;
-      cloud->points[depth_idx].intensity = static_cast<float> (pixel);
+      (*cloud)[depth_idx].x = static_cast<float> ((dist * s_theta * c_ksai) / 500.0 + 0.5);
+      (*cloud)[depth_idx].y = static_cast<float> ((dist * s_theta * s_ksai) / 500.0 + 0.5);
+      (*cloud)[depth_idx].z = static_cast<float> (dist * c_theta);
+      if ((*cloud)[depth_idx].z < 0.01f)
+        (*cloud)[depth_idx].z = 0.01f;
+      (*cloud)[depth_idx].z /= 500.0f;
+      (*cloud)[depth_idx].intensity = static_cast<float> (pixel);
 
       
       // Get rid of the noise
-      if(cloud->points[depth_idx].z > 0.8f || cloud->points[depth_idx].z < 0.02f)
+      if((*cloud)[depth_idx].z > 0.8f || (*cloud)[depth_idx].z < 0.02f)
       {
-        cloud->points[depth_idx].x = std::numeric_limits<float>::quiet_NaN ();
-      	cloud->points[depth_idx].y = std::numeric_limits<float>::quiet_NaN ();
-      	cloud->points[depth_idx].z = std::numeric_limits<float>::quiet_NaN ();
-        cloud->points[depth_idx].intensity = static_cast<float> (pixel);
+        (*cloud)[depth_idx].x = std::numeric_limits<float>::quiet_NaN ();
+      	(*cloud)[depth_idx].y = std::numeric_limits<float>::quiet_NaN ();
+      	(*cloud)[depth_idx].z = std::numeric_limits<float>::quiet_NaN ();
+        (*cloud)[depth_idx].intensity = static_cast<float> (pixel);
       }
     }
   }
@@ -361,18 +353,18 @@ pcl::DinastGrabber::USBRxControlData (const unsigned char req_code,
                                       int length)
 {
   // The direction of the transfer is inferred from the requesttype field of the setup packet.
-  unsigned char requesttype = (LIBUSB_RECIPIENT_DEVICE | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_IN);
+  const unsigned char requesttype = (static_cast<unsigned char>(LIBUSB_RECIPIENT_DEVICE) | static_cast<unsigned char>(LIBUSB_REQUEST_TYPE_VENDOR) | static_cast<unsigned char>(LIBUSB_ENDPOINT_IN));
   // The value field for the setup packet
-  uint16_t value = 0x02;
+  std::uint16_t value = 0x02;
   // The index field for the setup packet
-  uint16_t index = 0x08;
+  std::uint16_t index = 0x08;
   // timeout (in ms) that this function should wait before giving up due to no response being received
   // For an unlimited timeout, use value 0.
-  uint16_t timeout = 1000;
+  std::uint16_t timeout = 1000;
   
   int nr_read = libusb_control_transfer (device_handle_, requesttype,
-                                         req_code, value, index, buffer, static_cast<uint16_t> (length), timeout);
-  if (nr_read != int(length))
+                                         req_code, value, index, buffer, static_cast<std::uint16_t> (length), timeout);
+  if (nr_read != (length))
     PCL_THROW_EXCEPTION (pcl::IOException, "[pcl::DinastGrabber::USBRxControlData] Control data error");
 
   return (true);
@@ -385,18 +377,18 @@ pcl::DinastGrabber::USBTxControlData (const unsigned char req_code,
                                       int length)
 {
   // The direction of the transfer is inferred from the requesttype field of the setup packet.
-  unsigned char requesttype = (LIBUSB_RECIPIENT_DEVICE | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_ENDPOINT_OUT);
+  const unsigned char requesttype = (static_cast<unsigned char>(LIBUSB_RECIPIENT_DEVICE) | static_cast<unsigned char>(LIBUSB_REQUEST_TYPE_VENDOR) | static_cast<unsigned char>(LIBUSB_ENDPOINT_OUT));
   // The value field for the setup packet
-  uint16_t value = 0x01;
+  std::uint16_t value = 0x01;
   // The index field for the setup packet
-  uint16_t index = 0x00;
+  std::uint16_t index = 0x00;
   // timeout (in ms) that this function should wait before giving up due to no response being received
   // For an unlimited timeout, use value 0.
-  uint16_t timeout = 1000;
+  std::uint16_t timeout = 1000;
   
   int nr_read = libusb_control_transfer (device_handle_, requesttype,
-                                         req_code, value, index, buffer, static_cast<uint16_t> (length), timeout);
-  if (nr_read != int(length))
+                                         req_code, value, index, buffer, static_cast<std::uint16_t> (length), timeout);
+  if (nr_read != (length))
   {
     std::stringstream sstream;
     sstream << "[pcl::DinastGrabber::USBTxControlData] USB control data error, LIBUSB_ERROR: " << nr_read;
@@ -417,14 +409,14 @@ pcl::DinastGrabber::checkHeader ()
 
   int data_ptr = -1;
 
-  for (size_t i = 0; i < g_buffer_.size (); ++i)
+  for (std::size_t i = 0; i < g_buffer_.size (); ++i)
   {
     if ((g_buffer_[i + 0] == 0xAA) && (g_buffer_[i + 1] == 0xAA) &&
         (g_buffer_[i + 2] == 0x44) && (g_buffer_[i + 3] == 0x44) &&
         (g_buffer_[i + 4] == 0xBB) && (g_buffer_[i + 5] == 0xBB) &&
         (g_buffer_[i + 6] == 0x77) && (g_buffer_[i + 7] == 0x77))
     {
-      data_ptr = int (i) + sync_packet_size_;
+      data_ptr = static_cast<int>(i) + sync_packet_size_;
       break;
     }
   }
@@ -439,7 +431,7 @@ pcl::DinastGrabber::captureThreadFunction ()
   while (true)
   {
     // Lock before checking running flag
-    boost::unique_lock<boost::mutex> capture_lock (capture_mutex_);
+    std::unique_lock<std::mutex> capture_lock (capture_mutex_);
     if(running_)
     {
       readImage ();
