@@ -36,14 +36,10 @@
  *
  */
 
-#include <gtest/gtest.h>
-#include <pcl/common/common.h>
-#include <pcl/common/distances.h>
-#include <pcl/common/intersections.h>
-#include <pcl/common/io.h>
-#include <pcl/common/eigen.h>
+#include <pcl/test/gtest.h>
 #include <pcl/point_types.h>
 #include <pcl/point_cloud.h>
+#include <pcl/io/pcd_io.h>
 #include <pcl/pcl_tests.h>
 
 #include <pcl/common/centroid.h>
@@ -51,33 +47,40 @@
 using namespace pcl;
 using pcl::test::EXPECT_EQ_VECTORS;
 
+pcl::PCLPointCloud2 cloud_blob;
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, compute3DCentroidFloat)
 {
   pcl::PointIndices pindices;
-  std::vector<int> indices;
+  Indices indices;
   PointXYZ point;
   PointCloud<PointXYZ> cloud;
-  Eigen::Vector4f centroid;
+  Eigen::Vector4f centroid = Eigen::Vector4f::Random();
+  const Eigen::Vector4f old_centroid = centroid;
 
   // test empty cloud which is dense
   cloud.is_dense = true;
   EXPECT_EQ (compute3DCentroid (cloud, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   // test empty cloud non_dense
   cloud.is_dense = false;
   EXPECT_EQ (compute3DCentroid (cloud, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
-  // test non-empty cloud non_dense
+  // test non-empty cloud non_dense (with only invalid points)
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   EXPECT_EQ (compute3DCentroid (cloud, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
-  // test non-empty cloud non_dense
+  // test non-empty cloud non_dense (with only invalid points)
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   indices.push_back (1);
   EXPECT_EQ (compute3DCentroid (cloud, indices, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   cloud.clear ();
   indices.clear ();
@@ -161,29 +164,34 @@ TEST (PCL, compute3DCentroidFloat)
 TEST (PCL, compute3DCentroidDouble)
 {
   pcl::PointIndices pindices;
-  std::vector<int> indices;
+  Indices indices;
   PointXYZ point;
   PointCloud<PointXYZ> cloud;
-  Eigen::Vector4d centroid;
+  Eigen::Vector4d centroid = Eigen::Vector4d::Random();
+  const Eigen::Vector4d old_centroid = centroid;
 
   // test empty cloud which is dense
   cloud.is_dense = true;
   EXPECT_EQ (compute3DCentroid (cloud, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   // test empty cloud non_dense
   cloud.is_dense = false;
   EXPECT_EQ (compute3DCentroid (cloud, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
-  // test non-empty cloud non_dense
+  // test non-empty cloud non_dense (with only invalid points)
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   EXPECT_EQ (compute3DCentroid (cloud, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
-  // test non-empty cloud non_dense
+  // test non-empty cloud non_dense (with only invalid points)
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   indices.push_back (1);
   EXPECT_EQ (compute3DCentroid (cloud, indices, centroid), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   cloud.clear ();
   indices.clear ();
@@ -266,7 +274,7 @@ TEST (PCL, compute3DCentroidDouble)
 TEST (PCL, compute3DCentroidCloudIterator)
 {
   pcl::PointIndices pindices;
-  std::vector<int> indices;
+  Indices indices;
   PointXYZ point;
   PointCloud<PointXYZ> cloud;
   Eigen::Vector4f centroid_f;
@@ -321,6 +329,14 @@ TEST (PCL, compute3DCentroidCloudIterator)
 
     EXPECT_EQ (8, compute3DCentroid (it, centroid_f));
     EXPECT_EQ_VECTORS (Eigen::Vector4f (0.f, 0.f, 0.f, 1.f), centroid_f);
+
+    const Eigen::Vector4f old_centroid = centroid_f;
+    indices.clear ();
+    indices.push_back (cloud.size () - 1);
+    ConstCloudIterator<PointXYZ> it2 (cloud, indices);
+    // zero valid points and centroid remains unchanged
+    EXPECT_EQ (0, compute3DCentroid (it2, centroid_f));
+    EXPECT_EQ (old_centroid, centroid_f);
   }
 }
 
@@ -330,9 +346,10 @@ TEST (PCL, computeCovarianceMatrix)
 {
   PointCloud<PointXYZ> cloud;
   PointXYZ point;
-  std::vector <int> indices;
+  Indices indices;
   Eigen::Vector4f centroid;
-  Eigen::Matrix3f covariance_matrix;
+  Eigen::Matrix3f covariance_matrix = Eigen::Matrix3f::Random();
+  const Eigen::Matrix3f old_covariance_matrix = covariance_matrix;
 
   centroid [0] = 0;
   centroid [1] = 0;
@@ -341,21 +358,25 @@ TEST (PCL, computeCovarianceMatrix)
   // test empty cloud which is dense
   cloud.is_dense = true;
   EXPECT_EQ (computeCovarianceMatrix (cloud, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test empty cloud non_dense
   cloud.is_dense = false;
   EXPECT_EQ (computeCovarianceMatrix (cloud, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
-  // test non-empty cloud non_dense
+  // test non-empty cloud non_dense (with only invalid points)
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   EXPECT_EQ (computeCovarianceMatrix (cloud, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
-  // test non-empty cloud non_dense
+  // test non-empty cloud non_dense (with only invalid points)
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   indices.push_back (1);
   EXPECT_EQ (computeCovarianceMatrix (cloud, indices, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   cloud.clear ();
   indices.clear ();
@@ -446,9 +467,10 @@ TEST (PCL, computeCovarianceMatrixNormalized)
 {
   PointCloud<PointXYZ> cloud;
   PointXYZ point;
-  std::vector <int> indices;
+  Indices indices;
   Eigen::Vector4f centroid;
-  Eigen::Matrix3f covariance_matrix;
+  Eigen::Matrix3f covariance_matrix = Eigen::Matrix3f::Random();
+  const Eigen::Matrix3f old_covariance_matrix = covariance_matrix;
 
   centroid [0] = 0;
   centroid [1] = 0;
@@ -457,21 +479,25 @@ TEST (PCL, computeCovarianceMatrixNormalized)
   // test empty cloud which is dense
   cloud.is_dense = true;
   EXPECT_EQ (computeCovarianceMatrixNormalized (cloud, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test empty cloud non_dense
   cloud.is_dense = false;
   EXPECT_EQ (computeCovarianceMatrixNormalized (cloud, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test non-empty cloud non_dense
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   EXPECT_EQ (computeCovarianceMatrixNormalized (cloud, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test non-empty cloud non_dense
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   indices.push_back (1);
   EXPECT_EQ (computeCovarianceMatrixNormalized (cloud, indices, centroid, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   cloud.clear ();
   indices.clear ();
@@ -564,27 +590,32 @@ TEST (PCL, computeDemeanedCovariance)
 {
   PointCloud<PointXYZ> cloud;
   PointXYZ point;
-  std::vector <int> indices;
-  Eigen::Matrix3f covariance_matrix;
+  Indices indices;
+  Eigen::Matrix3f covariance_matrix = Eigen::Matrix3f::Random();
+  const Eigen::Matrix3f old_covariance_matrix = covariance_matrix;
 
   // test empty cloud which is dense
   cloud.is_dense = true;
   EXPECT_EQ (computeCovarianceMatrix (cloud, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test empty cloud non_dense
   cloud.is_dense = false;
   EXPECT_EQ (computeCovarianceMatrix (cloud, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test non-empty cloud non_dense
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   EXPECT_EQ (computeCovarianceMatrix (cloud, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   // test non-empty cloud non_dense
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   indices.push_back (1);
   EXPECT_EQ (computeCovarianceMatrix (cloud, indices, covariance_matrix), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
 
   cloud.clear ();
   indices.clear ();
@@ -670,28 +701,38 @@ TEST (PCL, computeMeanAndCovariance)
 {
   PointCloud<PointXYZ> cloud;
   PointXYZ point;
-  std::vector <int> indices;
-  Eigen::Matrix3f covariance_matrix;
-  Eigen::Vector4f centroid;
+  Indices indices;
+  Eigen::Matrix3f covariance_matrix = Eigen::Matrix3f::Random();
+  Eigen::Vector4f centroid = Eigen::Vector4f::Random();
+  const Eigen::Matrix3f old_covariance_matrix = covariance_matrix;
+  const Eigen::Vector4f old_centroid = centroid;
 
   // test empty cloud which is dense
   cloud.is_dense = true;
   EXPECT_EQ (computeMeanAndCovarianceMatrix (cloud, covariance_matrix, centroid), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   // test empty cloud non_dense
   cloud.is_dense = false;
   EXPECT_EQ (computeMeanAndCovarianceMatrix (cloud, covariance_matrix, centroid), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   // test non-empty cloud non_dense
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   EXPECT_EQ (computeMeanAndCovarianceMatrix (cloud, covariance_matrix, centroid), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   // test non-empty cloud non_dense
   point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
   cloud.push_back (point);
   indices.push_back (1);
   EXPECT_EQ (computeMeanAndCovarianceMatrix (cloud, indices, covariance_matrix, centroid), 0);
+  EXPECT_EQ (old_covariance_matrix, covariance_matrix); // cov. matrix remains unchanged
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
 
   cloud.clear ();
   indices.clear ();
@@ -796,6 +837,137 @@ TEST (PCL, computeMeanAndCovariance)
   EXPECT_EQ (covariance_matrix (2, 2), 1);
 }
 
+
+//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+TEST (PCL, computeCentroidAndOBB)
+{
+  PointCloud<PointXYZ> cloud;
+  PointXYZ point;
+  Indices indices;
+  Eigen::Vector3f centroid = Eigen::Vector3f::Random();
+  Eigen::Matrix<float, 3, 1> obb_position;
+  Eigen::Matrix<float, 3, 1> obb_dimensions;
+  Eigen::Matrix3f obb_rotational_matrix= Eigen::Matrix3f::Random();
+
+  const Eigen::Vector3f old_centroid = centroid;
+  const Eigen::Matrix3f old_obb_rotational_matrix= obb_rotational_matrix;
+
+  // test empty cloud which is dense
+  cloud.is_dense = true;
+  EXPECT_EQ (computeCentroidAndOBB (cloud, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
+  EXPECT_EQ (old_obb_rotational_matrix, obb_rotational_matrix); // obb_rotational_matrix remains unchanged
+
+  // test empty cloud non_dense
+  cloud.is_dense = false;
+  EXPECT_EQ (computeCentroidAndOBB (cloud, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
+  EXPECT_EQ (old_obb_rotational_matrix, obb_rotational_matrix); // obb_rotational_matrix remains unchanged
+
+  // test non-empty cloud non_dense with no valid points
+  point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
+  cloud.push_back (point);
+  EXPECT_EQ (computeCentroidAndOBB (cloud, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
+  EXPECT_EQ (old_obb_rotational_matrix, obb_rotational_matrix); // obb_rotational_matrix remains unchanged
+
+  // test non-empty cloud non_dense with no valid points
+  point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
+  cloud.push_back (point);
+  indices.push_back (1);
+  EXPECT_EQ (computeCentroidAndOBB (cloud, indices, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 0);
+  EXPECT_EQ (old_centroid, centroid); // centroid remains unchanged
+  EXPECT_EQ (old_obb_rotational_matrix, obb_rotational_matrix); // obb_rotational_matrix remains unchanged
+
+  cloud.clear ();
+  indices.clear ();
+  // -1 -1 -1 / -1 -1 1 / -1 1 -1 / -1 1 1 / 1 -1 -1 / 1 -1 1 / 1 1 -1 / 1 1 1
+  for (point.x = -1; point.x < 2; point.x += 2)
+  {
+    for (point.y = -1; point.y < 2; point.y += 2)
+    {
+      for (point.z = -1; point.z < 2; point.z += 2)
+      {
+        cloud.push_back (point);
+      }
+    }
+  }
+  cloud.is_dense = true;
+
+  // eight points with (0, 0, 0) as centroid
+
+  centroid [0] = -100;
+  centroid [1] = -101;
+  centroid [2] = -102;
+  EXPECT_EQ (computeCentroidAndOBB (cloud, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 8);
+  EXPECT_EQ (centroid [0], 0);
+  EXPECT_EQ (centroid [1], 0);
+  EXPECT_EQ (centroid [2], 0);
+  EXPECT_NEAR (obb_position(0), 0, 0.01);
+  EXPECT_NEAR (obb_position(1), 0, 0.01);
+  EXPECT_NEAR (obb_position(2), 0, 0.01);
+  EXPECT_NEAR (obb_dimensions(0), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(1), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(2), 2, 0.01);
+
+
+  indices.resize (4); // only positive y values
+  indices [0] = 2;
+  indices [1] = 3;
+  indices [2] = 6;
+  indices [3] = 7;
+  centroid [0] = -100;
+  centroid [1] = -101;
+  centroid [2] = -102;
+
+  EXPECT_EQ (computeCentroidAndOBB (cloud, indices, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 4);
+  EXPECT_EQ (centroid [0], 0);
+  EXPECT_NEAR (centroid [1], 1, 0.001);
+  EXPECT_EQ (centroid [2], 0);
+  EXPECT_NEAR (obb_position(0), 0, 0.01);
+  EXPECT_NEAR (obb_position(1), 1, 0.01);
+  EXPECT_NEAR (obb_position(2), 0, 0.01);
+  EXPECT_NEAR (obb_dimensions(0), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(1), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(2), 0, 0.01);
+
+
+  point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
+  cloud.push_back (point);
+  cloud.is_dense = false;
+  centroid [0] = -100;
+  centroid [1] = -101;
+  centroid [2] = -102;
+  EXPECT_EQ (computeCentroidAndOBB (cloud, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 8);
+  EXPECT_EQ (centroid [0], 0);
+  EXPECT_EQ (centroid [1], 0);
+  EXPECT_EQ (centroid [2], 0);
+  EXPECT_NEAR (obb_position(0), 0, 0.01);
+  EXPECT_NEAR (obb_position(1), 0, 0.01);
+  EXPECT_NEAR (obb_position(2), 0, 0.01);
+  EXPECT_NEAR (obb_dimensions(0), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(1), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(2), 2, 0.01);
+
+
+  indices.push_back (8); // add the NaN
+  centroid [0] = -100;
+  centroid [1] = -101;
+  centroid [2] = -102;
+
+  EXPECT_EQ (computeCentroidAndOBB (cloud, indices, centroid, obb_position, obb_dimensions, obb_rotational_matrix), 4);
+  EXPECT_EQ (centroid [0], 0);
+  EXPECT_NEAR (centroid [1], 1, 0.001);
+  EXPECT_EQ (centroid [2], 0);
+  EXPECT_NEAR (obb_position(0), 0, 0.01);
+  EXPECT_NEAR (obb_position(1), 1, 0.01);
+  EXPECT_NEAR (obb_position(2), 0, 0.01);
+  EXPECT_NEAR (obb_dimensions(0), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(1), 2, 0.01);
+  EXPECT_NEAR (obb_dimensions(2), 0, 0.01);
+
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, CentroidPoint)
 {
@@ -810,6 +982,7 @@ TEST (PCL, CentroidPoint)
     centroid.get (c);
     EXPECT_XYZ_EQ (PointXYZ (100, 100, 100), c);
   }
+
   // Single point
   {
     CentroidPoint<PointXYZ> centroid;
@@ -818,6 +991,7 @@ TEST (PCL, CentroidPoint)
     centroid.get (c);
     EXPECT_XYZ_EQ (p1, c);
   }
+
   // Multiple points
   {
     CentroidPoint<PointXYZ> centroid;
@@ -831,12 +1005,36 @@ TEST (PCL, CentroidPoint)
 
   // Retrieve centroid into a different point type
   {
-    CentroidPoint<PointXYZ> centroid;
+    PointNormal p1; p1.getVector3fMap () << 1, 2, 3; p1.getNormalVector3fMap() << 0, 1, 0;
+    CentroidPoint<PointNormal> centroid;
     centroid.add (p1);
-    PointXYZRGB c; c.rgba = 0x00FFFFFF;
-    centroid.get (c);
-    EXPECT_XYZ_EQ (p1, c);
-    EXPECT_EQ (0x00FFFFFF, c.rgba);
+    // Retrieve into a point type that is a "superset" of the original
+    {
+      PointXYZRGBNormal c; c.rgba = 0x00FFFFFF;
+      centroid.get (c);
+      EXPECT_XYZ_EQ (p1, c);
+      EXPECT_NORMAL_EQ (p1, c);
+      EXPECT_EQ (0x00FFFFFF, c.rgba); // unchanged
+    }
+    // Retrieve into a point type that is a "subset" of the original
+    {
+      Normal c;
+      centroid.get (c);
+      EXPECT_NORMAL_EQ (p1, c);
+    }
+    // Retrieve into a point type that partially "overlaps" with the original
+    {
+      PointXYZRGB c; c.rgba = 0x00FFFFFF;
+      centroid.get (c);
+      EXPECT_XYZ_EQ (p1, c);
+      EXPECT_EQ (0x00FFFFFF, c.rgba); // unchanged
+    }
+    // Retrieve into a point type that does not "overlap" with the original
+    {
+      RGB c; c.rgba = 0x00FFFFFF;
+      centroid.get (c);
+      EXPECT_EQ (0x00FFFFFF, c.rgba); // unchanged
+    }
   }
 
   // Centroid with XYZ and RGB
@@ -854,7 +1052,7 @@ TEST (PCL, CentroidPoint)
     EXPECT_EQ (0xFF111111, c.rgba);
   }
 
-  // Centroid with normal and curavture
+  // Centroid with normal and curvature
   {
     Normal np1; np1.getNormalVector4fMap () << 1, 0, 0, 0; np1.curvature = 0.2;
     Normal np2; np2.getNormalVector4fMap () << -1, 0, 0, 0; np2.curvature = 0.1;
@@ -902,7 +1100,7 @@ TEST (PCL, CentroidPoint)
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 TEST (PCL, computeCentroid)
 {
-  std::vector<int> indices;
+  Indices indices;
   PointXYZI point;
   PointCloud<PointXYZI> cloud;
   PointXYZINormal centroid;
@@ -1009,9 +1207,80 @@ TEST (PCL, computeCentroid)
   EXPECT_FLOAT_EQ (-500, centroid.curvature);
 }
 
+TEST (PCL, demeanPointCloud)
+{
+  PointCloud<PointXYZ> cloud, cloud_demean;
+  fromPCLPointCloud2 (cloud_blob, cloud);
+
+  Eigen::Vector4f centroid;
+  compute3DCentroid (cloud, centroid);
+  EXPECT_NEAR (centroid[0], -0.0290809, 1e-4);
+  EXPECT_NEAR (centroid[1],  0.102653,  1e-4);
+  EXPECT_NEAR (centroid[2],  0.027302,  1e-4);
+  EXPECT_NEAR (centroid[3],  1,         1e-4);
+
+  // Check standard demean
+  demeanPointCloud (cloud, centroid, cloud_demean);
+  EXPECT_METADATA_EQ (cloud_demean, cloud);
+
+  EXPECT_XYZ_NEAR (cloud_demean[0], PointXYZ (0.034503, 0.010837, 0.013447), 1e-4);
+  EXPECT_XYZ_NEAR (cloud_demean[cloud_demean.size () - 1], PointXYZ (-0.048849, 0.072507, -0.071702), 1e-4);
+
+  Indices indices (cloud.size ());
+  for (int i = 0; i < static_cast<int> (indices.size ()); ++i) { indices[i] = i; }
+
+  // Check standard demean w/ indices
+  demeanPointCloud (cloud, indices, centroid, cloud_demean);
+  EXPECT_EQ (cloud_demean.is_dense, cloud.is_dense);
+  EXPECT_EQ (cloud_demean.width, indices.size ());
+  EXPECT_EQ (cloud_demean.height, 1);
+  EXPECT_EQ (cloud_demean.size (), cloud.size ());
+
+  EXPECT_XYZ_NEAR (cloud_demean[0], PointXYZ (0.034503, 0.010837, 0.013447), 1e-4);
+  EXPECT_XYZ_NEAR (cloud_demean[cloud_demean.size () - 1], PointXYZ (-0.048849, 0.072507, -0.071702), 1e-4);
+
+  // Check eigen demean
+  Eigen::MatrixXf mat_demean;
+  demeanPointCloud (cloud, centroid, mat_demean);
+  EXPECT_EQ (mat_demean.cols (), int (cloud.size ()));
+  EXPECT_EQ (mat_demean.rows (), 4);
+
+  EXPECT_NEAR (mat_demean (0, 0), 0.034503, 1e-4);
+  EXPECT_NEAR (mat_demean (1, 0), 0.010837, 1e-4);
+  EXPECT_NEAR (mat_demean (2, 0), 0.013447, 1e-4);
+
+  EXPECT_NEAR (mat_demean (0, cloud_demean.size () - 1), -0.048849, 1e-4);
+  EXPECT_NEAR (mat_demean (1, cloud_demean.size () - 1),  0.072507, 1e-4);
+  EXPECT_NEAR (mat_demean (2, cloud_demean.size () - 1), -0.071702, 1e-4);
+
+  // Check eigen demean + indices
+  demeanPointCloud (cloud, indices, centroid, mat_demean);
+  EXPECT_EQ (mat_demean.cols (), int (cloud.size ()));
+  EXPECT_EQ (mat_demean.rows (), 4);
+
+  EXPECT_NEAR (mat_demean (0, 0), 0.034503, 1e-4);
+  EXPECT_NEAR (mat_demean (1, 0), 0.010837, 1e-4);
+  EXPECT_NEAR (mat_demean (2, 0), 0.013447, 1e-4);
+
+  EXPECT_NEAR (mat_demean (0, cloud_demean.size () - 1), -0.048849, 1e-4);
+  EXPECT_NEAR (mat_demean (1, cloud_demean.size () - 1),  0.072507, 1e-4);
+  EXPECT_NEAR (mat_demean (2, cloud_demean.size () - 1), -0.071702, 1e-4);
+}
+
 int
 main (int argc, char** argv)
 {
+  if (argc < 2)
+  {
+    std::cerr << "No test file given. Please download `bun0.pcd` and pass its path to the test." << std::endl;
+    return (-1);
+  }
+  if (io::loadPCDFile (argv[1], cloud_blob) < 0)
+  {
+    std::cerr << "Failed to read test file. Please download `bun0.pcd` and pass its path to the test." << std::endl;
+    return (-1);
+  }
+
   testing::InitGoogleTest (&argc, argv);
   return (RUN_ALL_TESTS ());
 }
