@@ -1,94 +1,89 @@
-#include <gtest/gtest.h>
+#include <pcl/test/gtest.h>
+#include <pcl/common/pcl_filesystem.h>
 #include <pcl/point_types.h>
-#include <pcl/common/io.h>
 #include <pcl/io/pcd_io.h>
 #include <pcl/io/pcd_grabber.h>
 #include <pcl/io/image_grabber.h>
-#include <pcl/console/time.h>
 
+#include <chrono>
 #include <string>
+#include <thread>
 #include <vector>
 
-using namespace std;
+#include <boost/algorithm/string/case_conv.hpp> // for to_upper_copy
 
-typedef pcl::PointXYZRGBA PointT;
-typedef pcl::PointCloud<PointT> CloudT;
+using namespace std::chrono_literals;
 
-string tiff_dir_;
-string pclzf_dir_;
-string pcd_dir_;
-vector<CloudT::ConstPtr> pcds_;
-vector<std::string> pcd_files_;
+using PointT = pcl::PointXYZRGBA;
+using CloudT = pcl::PointCloud<PointT>;
+
+std::string tiff_dir_;
+std::string pclzf_dir_;
+std::string pcd_dir_;
+std::vector<CloudT::ConstPtr> pcds_;
+std::vector<std::string> pcd_files_;
 
 
 // Helper function for grabbing a cloud
 void
-cloud_callback (bool *signal_received, 
-                CloudT::ConstPtr *ptr_to_fill, 
-                const CloudT::ConstPtr &input_cloud)
+cloud_callback (bool& signal_received,
+                CloudT::ConstPtr& ptr_to_fill,
+                const CloudT::ConstPtr& input_cloud)
 {
-  *signal_received = true;
-  *ptr_to_fill = input_cloud;
-}
-
-// Helper function for grabbing a cloud (vector
-void
-cloud_callback_vector (std::vector<CloudT::ConstPtr> *vector_to_fill, 
-                       const CloudT::ConstPtr &input_cloud)
-{
-  vector_to_fill->push_back (input_cloud);
+  signal_received = true;
+  ptr_to_fill = input_cloud;
 }
 
 TEST (PCL, PCDGrabber)
 {
   pcl::PCDGrabber<PointT> grabber (pcd_files_, 10, false); // TODO add directory functionality
   EXPECT_EQ (grabber.size (), pcds_.size ());
-  vector<CloudT::ConstPtr> grabbed_clouds;
-  boost::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)> 
-    fxn = boost::bind (cloud_callback_vector, &grabbed_clouds, _1);
+  std::vector<CloudT::ConstPtr> grabbed_clouds;
+  std::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)>
+    fxn = [&] (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr& input_cloud) { grabbed_clouds.push_back (input_cloud); };
   grabber.registerCallback (fxn);
   grabber.start ();
   // 1 second should be /plenty/ of time
-  boost::this_thread::sleep (boost::posix_time::microseconds (1E6));
+  std::this_thread::sleep_for(1s);
   grabber.stop ();
 
   //// Make sure they match
   ASSERT_EQ (pcds_.size (), grabbed_clouds.size ());
-  for (size_t i = 0; i < pcds_.size (); i++)
+  for (std::size_t i = 0; i < pcds_.size (); i++)
   {
     // Also compare against FileGrabber-style loaded cloud
     CloudT::ConstPtr cloud_from_file_grabber = grabber[i];
     EXPECT_EQ (grabbed_clouds[i]->size (), pcds_[i]->size ());
     EXPECT_EQ (cloud_from_file_grabber->size (), pcds_[i]->size ());
-    for (size_t j = 0; j < pcds_[i]->size (); j++)
+    for (std::size_t j = 0; j < pcds_[i]->size (); j++)
     {
       const PointT &pcd_pt = pcds_[i]->at(j);
       const PointT &grabbed_pt = grabbed_clouds[i]->at(j);
       const PointT &fg_pt = cloud_from_file_grabber->at(j);
-      if (pcl_isnan (pcd_pt.x))
+      if (std::isnan (pcd_pt.x))
       {
-        EXPECT_TRUE (pcl_isnan (grabbed_pt.x));
-        EXPECT_TRUE (pcl_isnan (fg_pt.x));
+        EXPECT_TRUE (std::isnan (grabbed_pt.x));
+        EXPECT_TRUE (std::isnan (fg_pt.x));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.x, grabbed_pt.x);
         EXPECT_FLOAT_EQ (pcd_pt.x, fg_pt.x);
       }
-      if (pcl_isnan (pcd_pt.y))
+      if (std::isnan (pcd_pt.y))
       {
-        EXPECT_TRUE (pcl_isnan (grabbed_pt.y));
-        EXPECT_TRUE (pcl_isnan (fg_pt.y));
+        EXPECT_TRUE (std::isnan (grabbed_pt.y));
+        EXPECT_TRUE (std::isnan (fg_pt.y));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.y, grabbed_pt.y);
         EXPECT_FLOAT_EQ (pcd_pt.y, fg_pt.y);
       }
-      if (pcl_isnan (pcd_pt.z))
+      if (std::isnan (pcd_pt.z))
       {
-        EXPECT_TRUE (pcl_isnan (grabbed_pt.z));
-        EXPECT_TRUE (pcl_isnan (fg_pt.z));
+        EXPECT_TRUE (std::isnan (grabbed_pt.z));
+        EXPECT_TRUE (std::isnan (fg_pt.z));
       }
       else
       {
@@ -112,24 +107,26 @@ TEST (PCL, ImageGrabberTIFF)
 {
   // Get all clouds from the grabber
   pcl::ImageGrabber<PointT> grabber (tiff_dir_, 0, false, false);
-  vector<CloudT::ConstPtr> tiff_clouds;
+  std::vector<CloudT::ConstPtr> tiff_clouds;
   CloudT::ConstPtr cloud_buffer;
   bool signal_received = false;
-  boost::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)> 
-    fxn = boost::bind (cloud_callback, &signal_received, &cloud_buffer, _1);
+  std::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)>
+    fxn = [&] (const CloudT::ConstPtr& input_cloud) { cloud_callback (signal_received, cloud_buffer, input_cloud); };
   grabber.registerCallback (fxn);
   grabber.setCameraIntrinsics (525., 525., 320., 240.); // Setting old intrinsics which were used to generate these tests
   grabber.start ();
-  for (size_t i = 0; i < grabber.size (); i++)
+  for (std::size_t i = 0; i < grabber.size (); i++)
   {
     grabber.trigger ();
-    size_t niter = 0;
+    std::size_t niter = 0;
     while (!signal_received)
     {
-      boost::this_thread::sleep (boost::posix_time::microseconds (10000));
+      std::this_thread::sleep_for(10ms);
       if (++niter > 100)
       {
-        ASSERT_TRUE (false);
+        #ifdef PCL_BUILT_WITH_VTK
+          FAIL ();
+        #endif
         return;
       }
     }
@@ -139,39 +136,39 @@ TEST (PCL, ImageGrabberTIFF)
 
   // Make sure they match
   EXPECT_EQ (pcds_.size (), tiff_clouds.size ());
-  for (size_t i = 0; i < pcds_.size (); i++)
+  for (std::size_t i = 0; i < pcds_.size (); i++)
   {
     // Also compare against dynamically loaded cloud
     CloudT::ConstPtr cloud_from_file_grabber = grabber[i];
-    for (size_t j = 0; j < pcds_[i]->size (); j++)
+    for (std::size_t j = 0; j < pcds_[i]->size (); j++)
     {
       const PointT &pcd_pt = pcds_[i]->at(j);
       const PointT &tiff_pt = tiff_clouds[i]->at(j);
       const PointT &tiff_fg_pt = cloud_from_file_grabber->at(j);
-      if (pcl_isnan (pcd_pt.x))
+      if (std::isnan (pcd_pt.x))
       {
-        EXPECT_TRUE (pcl_isnan (tiff_pt.x));
-        EXPECT_TRUE (pcl_isnan (tiff_fg_pt.x));
+        EXPECT_TRUE (std::isnan (tiff_pt.x));
+        EXPECT_TRUE (std::isnan (tiff_fg_pt.x));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.x, tiff_pt.x);
         EXPECT_FLOAT_EQ (pcd_pt.x, tiff_fg_pt.x);
       }
-      if (pcl_isnan (pcd_pt.y))
+      if (std::isnan (pcd_pt.y))
       {
-        EXPECT_TRUE (pcl_isnan (tiff_pt.y));
-        EXPECT_TRUE (pcl_isnan (tiff_fg_pt.y));
+        EXPECT_TRUE (std::isnan (tiff_pt.y));
+        EXPECT_TRUE (std::isnan (tiff_fg_pt.y));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.y, tiff_pt.y);
         EXPECT_FLOAT_EQ (pcd_pt.y, tiff_fg_pt.y);
       }
-      if (pcl_isnan (pcd_pt.z))
+      if (std::isnan (pcd_pt.z))
       {
-        EXPECT_TRUE (pcl_isnan (tiff_pt.z));
-        EXPECT_TRUE (pcl_isnan (tiff_fg_pt.z));
+        EXPECT_TRUE (std::isnan (tiff_pt.z));
+        EXPECT_TRUE (std::isnan (tiff_fg_pt.z));
       }
       else
       {
@@ -193,20 +190,20 @@ TEST (PCL, ImageGrabberPCLZF)
 {
   // Get all clouds from the grabber
   pcl::ImageGrabber<PointT> grabber (pclzf_dir_, 0, false, true);
-  vector <CloudT::ConstPtr> pclzf_clouds;
+  std::vector <CloudT::ConstPtr> pclzf_clouds;
   CloudT::ConstPtr cloud_buffer;
   bool signal_received = false;
-  boost::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)> 
-    fxn = boost::bind (cloud_callback, &signal_received, &cloud_buffer, _1);
+  std::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)>
+    fxn = [&] (const CloudT::ConstPtr& input_cloud) { cloud_callback (signal_received, cloud_buffer, input_cloud); };
   grabber.registerCallback (fxn);
   grabber.start ();
-  for (size_t i = 0; i < grabber.size (); i++)
+  for (std::size_t i = 0; i < grabber.size (); i++)
   {
     grabber.trigger ();
-    size_t niter = 0;
+    std::size_t niter = 0;
     while (!signal_received)
     {
-      boost::this_thread::sleep (boost::posix_time::microseconds (10000));
+      std::this_thread::sleep_for(10ms);
       if (++niter > 100)
       {
         ASSERT_TRUE (false);
@@ -219,38 +216,38 @@ TEST (PCL, ImageGrabberPCLZF)
   // Make sure they match
   EXPECT_EQ (pcds_.size (), pclzf_clouds.size ());
   EXPECT_EQ (pcds_.size (), grabber.size ());
-  for (size_t i = 0; i < pcds_.size (); i++)
+  for (std::size_t i = 0; i < pcds_.size (); i++)
   {
     CloudT::ConstPtr cloud_from_file_grabber = grabber[i];
-    for (size_t j = 0; j < pcds_[i]->size (); j++)
+    for (std::size_t j = 0; j < pcds_[i]->size (); j++)
     {
       const PointT &pcd_pt = pcds_[i]->at(j);
       const PointT &pclzf_pt = pclzf_clouds[i]->at(j);
       const PointT &pclzf_fg_pt = cloud_from_file_grabber->at(j);
-      if (pcl_isnan (pcd_pt.x))
+      if (std::isnan (pcd_pt.x))
       {
-        EXPECT_TRUE (pcl_isnan (pclzf_pt.x));
-        EXPECT_TRUE (pcl_isnan (pclzf_fg_pt.x));
+        EXPECT_TRUE (std::isnan (pclzf_pt.x));
+        EXPECT_TRUE (std::isnan (pclzf_fg_pt.x));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.x, pclzf_pt.x);
         EXPECT_FLOAT_EQ (pcd_pt.x, pclzf_fg_pt.x);
       }
-      if (pcl_isnan (pcd_pt.y))
+      if (std::isnan (pcd_pt.y))
       {
-        EXPECT_TRUE (pcl_isnan (pclzf_pt.y));
-        EXPECT_TRUE (pcl_isnan (pclzf_fg_pt.y));
+        EXPECT_TRUE (std::isnan (pclzf_pt.y));
+        EXPECT_TRUE (std::isnan (pclzf_fg_pt.y));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.y, pclzf_pt.y);
         EXPECT_FLOAT_EQ (pcd_pt.y, pclzf_fg_pt.y);
       }
-      if (pcl_isnan (pcd_pt.z))
+      if (std::isnan (pcd_pt.z))
       {
-        EXPECT_TRUE (pcl_isnan (pclzf_pt.z));
-        EXPECT_TRUE (pcl_isnan (pclzf_fg_pt.z));
+        EXPECT_TRUE (std::isnan (pclzf_pt.z));
+        EXPECT_TRUE (std::isnan (pclzf_fg_pt.z));
       }
       else
       {
@@ -272,21 +269,21 @@ TEST (PCL, ImageGrabberOMP)
 {
   // Get all clouds from the grabber
   pcl::ImageGrabber<PointT> grabber (pclzf_dir_, 0, false, true);
-  vector <CloudT::ConstPtr> pclzf_clouds;
+  std::vector <CloudT::ConstPtr> pclzf_clouds;
   CloudT::ConstPtr cloud_buffer;
   bool signal_received = false;
-  boost::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)> 
-    fxn = boost::bind (cloud_callback, &signal_received, &cloud_buffer, _1);
+  std::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)>
+    fxn = [&] (const CloudT::ConstPtr& input_cloud) { cloud_callback (signal_received, cloud_buffer, input_cloud); };
   grabber.registerCallback (fxn);
   grabber.setNumberOfThreads (0); // Let OMP select
   grabber.start ();
-  for (size_t i = 0; i < grabber.size (); i++)
+  for (std::size_t i = 0; i < grabber.size (); i++)
   {
     grabber.trigger ();
-    size_t niter = 0;
+    std::size_t niter = 0;
     while (!signal_received)
     {
-      boost::this_thread::sleep (boost::posix_time::microseconds (10000));
+      std::this_thread::sleep_for(10ms);
       if (++niter > 100)
       {
         ASSERT_TRUE (false);
@@ -299,38 +296,38 @@ TEST (PCL, ImageGrabberOMP)
   // Make sure they match
   EXPECT_EQ (pcds_.size (), pclzf_clouds.size ());
   EXPECT_EQ (pcds_.size (), grabber.size ());
-  for (size_t i = 0; i < pcds_.size (); i++)
+  for (std::size_t i = 0; i < pcds_.size (); i++)
   {
     CloudT::ConstPtr cloud_from_file_grabber = grabber[i];
-    for (size_t j = 0; j < pcds_[i]->size (); j++)
+    for (std::size_t j = 0; j < pcds_[i]->size (); j++)
     {
       const PointT &pcd_pt = pcds_[i]->at(j);
       const PointT &pclzf_pt = pclzf_clouds[i]->at(j);
       const PointT &pclzf_fg_pt = cloud_from_file_grabber->at(j);
-      if (pcl_isnan (pcd_pt.x))
+      if (std::isnan (pcd_pt.x))
       {
-        EXPECT_TRUE (pcl_isnan (pclzf_pt.x));
-        EXPECT_TRUE (pcl_isnan (pclzf_fg_pt.x));
+        EXPECT_TRUE (std::isnan (pclzf_pt.x));
+        EXPECT_TRUE (std::isnan (pclzf_fg_pt.x));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.x, pclzf_pt.x);
         EXPECT_FLOAT_EQ (pcd_pt.x, pclzf_fg_pt.x);
       }
-      if (pcl_isnan (pcd_pt.y))
+      if (std::isnan (pcd_pt.y))
       {
-        EXPECT_TRUE (pcl_isnan (pclzf_pt.y));
-        EXPECT_TRUE (pcl_isnan (pclzf_fg_pt.y));
+        EXPECT_TRUE (std::isnan (pclzf_pt.y));
+        EXPECT_TRUE (std::isnan (pclzf_fg_pt.y));
       }
       else
       {
         EXPECT_FLOAT_EQ (pcd_pt.y, pclzf_pt.y);
         EXPECT_FLOAT_EQ (pcd_pt.y, pclzf_fg_pt.y);
       }
-      if (pcl_isnan (pcd_pt.z))
+      if (std::isnan (pcd_pt.z))
       {
-        EXPECT_TRUE (pcl_isnan (pclzf_pt.z));
-        EXPECT_TRUE (pcl_isnan (pclzf_fg_pt.z));
+        EXPECT_TRUE (std::isnan (pclzf_pt.z));
+        EXPECT_TRUE (std::isnan (pclzf_fg_pt.z));
       }
       else
       {
@@ -352,11 +349,11 @@ TEST (PCL, ImageGrabberTimestamps)
 {
   // Initialize the grabber but don't load
   pcl::ImageGrabber<PointT> grabber (pclzf_dir_, 0, false, true);
-  uint64_t frame0_microsec, frame1_microsec; 
+  std::uint64_t frame0_microsec, frame1_microsec; 
   ASSERT_EQ (grabber.size (), 3);
   EXPECT_TRUE (grabber.getTimestampAtIndex (0, frame0_microsec));
   EXPECT_TRUE (grabber.getTimestampAtIndex (1, frame1_microsec));
-  uint64_t timediff = frame1_microsec - frame0_microsec;
+  std::uint64_t timediff = frame1_microsec - frame0_microsec;
   EXPECT_EQ (timediff, 254471); // 20121214T142256.068683 - 20121214T142255.814212 
 }
 
@@ -365,11 +362,11 @@ TEST (PCL, ImageGrabberSetIntrinsicsTIFF)
   pcl::ImageGrabber<PointT> grabber (tiff_dir_, 0, false, false);
 
   // Get all clouds from the grabber
-  vector <CloudT::ConstPtr> tiff_clouds;
+  std::vector <CloudT::ConstPtr> tiff_clouds;
   CloudT::ConstPtr cloud_buffer;
   bool signal_received = false;
-  boost::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)> 
-    fxn = boost::bind (cloud_callback, &signal_received, &cloud_buffer, _1);
+  std::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)>
+    fxn = [&] (const CloudT::ConstPtr& input_cloud) { cloud_callback (signal_received, cloud_buffer, input_cloud); };
   grabber.registerCallback (fxn);
   grabber.start ();
   // Change the camera parameters
@@ -389,16 +386,18 @@ TEST (PCL, ImageGrabberSetIntrinsicsTIFF)
   double cy_new = cy_multiplier * cy_old;
   grabber.setCameraIntrinsics (fx_new, fy_new, cx_new, cy_new);
   // Collect the clouds
-  for (size_t i = 0; i < grabber.size (); i++)
+  for (std::size_t i = 0; i < grabber.size (); i++)
   {
     grabber.trigger ();
-    size_t niter = 0;
+    std::size_t niter = 0;
     while (!signal_received)
     {
-      boost::this_thread::sleep (boost::posix_time::microseconds (10000));
+      std::this_thread::sleep_for(10ms);
       if (++niter > 100)
       {
-        ASSERT_TRUE (false);
+        #ifdef PCL_BUILT_WITH_VTK
+          FAIL ();
+        #endif
         return;
       }
     }
@@ -408,26 +407,26 @@ TEST (PCL, ImageGrabberSetIntrinsicsTIFF)
 
   // Make sure they match
   EXPECT_EQ (pcds_.size (), tiff_clouds.size ());
-  for (size_t i = 0; i < pcds_.size (); i++)
+  for (std::size_t i = 0; i < pcds_.size (); i++)
   {
     EXPECT_EQ (pcds_[i]->width, tiff_clouds[i]->width);
     EXPECT_EQ (pcds_[i]->height, tiff_clouds[i]->height);
-    for (uint32_t x = 0; x < pcds_[i]->width; x++)
+    for (std::uint32_t x = 0; x < pcds_[i]->width; x++)
     {
-      for (uint32_t y = 0; y < pcds_[i]->height; y++)
+      for (std::uint32_t y = 0; y < pcds_[i]->height; y++)
       {
         const PointT &pcd_pt = pcds_[i]->operator()(x,y);
         const PointT &tiff_pt = tiff_clouds[i]->operator()(x,y);
-        if (pcl_isnan (pcd_pt.x))
-          EXPECT_TRUE (pcl_isnan (tiff_pt.x));
+        if (std::isnan (pcd_pt.x))
+          EXPECT_TRUE (std::isnan (tiff_pt.x));
         else
           EXPECT_NEAR ( pcd_pt.x * (x-cx_new), tiff_pt.x * fx_multiplier * (x-cx_old), 1E-4);
-        if (pcl_isnan (pcd_pt.y))
-          EXPECT_TRUE (pcl_isnan (tiff_pt.y));
+        if (std::isnan (pcd_pt.y))
+          EXPECT_TRUE (std::isnan (tiff_pt.y));
         else
           EXPECT_NEAR ( pcd_pt.y * (y-cy_new), tiff_pt.y * fy_multiplier * (y-cy_old), 1E-4);
-        if (pcl_isnan (pcd_pt.z))
-          EXPECT_TRUE (pcl_isnan (tiff_pt.z));
+        if (std::isnan (pcd_pt.z))
+          EXPECT_TRUE (std::isnan (tiff_pt.z));
         else
           EXPECT_FLOAT_EQ (pcd_pt.z, tiff_pt.z);
       }
@@ -441,11 +440,11 @@ TEST (PCL, ImageGrabberSetIntrinsicsPCLZF)
   pcl::ImageGrabber<PointT> grabber (pclzf_dir_, 0, false, true);
 
   // Get all clouds from the grabber
-  vector <CloudT::ConstPtr> pclzf_clouds;
+  std::vector <CloudT::ConstPtr> pclzf_clouds;
   CloudT::ConstPtr cloud_buffer;
   bool signal_received = false;
-  boost::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)> 
-    fxn = boost::bind (cloud_callback, &signal_received, &cloud_buffer, _1);
+  std::function<void (const pcl::PointCloud<pcl::PointXYZRGBA>::ConstPtr&)>
+    fxn = [&] (const CloudT::ConstPtr& input_cloud) { cloud_callback (signal_received, cloud_buffer, input_cloud); };
   grabber.registerCallback (fxn);
   grabber.start ();
   // Change the camera parameters
@@ -461,13 +460,13 @@ TEST (PCL, ImageGrabberSetIntrinsicsPCLZF)
   double cy_new = cy_multiplier * cy_old;
   grabber.setCameraIntrinsics (fx_new, fy_new, cx_new, cy_new);
   // Collect the clouds
-  for (size_t i = 0; i < grabber.size (); i++)
+  for (std::size_t i = 0; i < grabber.size (); i++)
   {
     grabber.trigger ();
-    size_t niter = 0;
+    std::size_t niter = 0;
     while (!signal_received)
     {
-      boost::this_thread::sleep (boost::posix_time::microseconds (10000));
+      std::this_thread::sleep_for(10ms);
       if (++niter > 100)
       {
         ASSERT_TRUE (false);
@@ -480,26 +479,26 @@ TEST (PCL, ImageGrabberSetIntrinsicsPCLZF)
 
   // Make sure they match
   EXPECT_EQ (pcds_.size (), pclzf_clouds.size ());
-  for (size_t i = 0; i < pcds_.size (); i++)
+  for (std::size_t i = 0; i < pcds_.size (); i++)
   {
     EXPECT_EQ (pcds_[i]->width, pclzf_clouds[i]->width);
     EXPECT_EQ (pcds_[i]->height, pclzf_clouds[i]->height);
-    for (uint32_t x = 0; x < pcds_[i]->width; x++)
+    for (std::uint32_t x = 0; x < pcds_[i]->width; x++)
     {
-      for (uint32_t y = 0; y < pcds_[i]->height; y++)
+      for (std::uint32_t y = 0; y < pcds_[i]->height; y++)
       {
         const PointT &pcd_pt = pcds_[i]->operator()(x,y);
         const PointT &pclzf_pt = pclzf_clouds[i]->operator()(x,y);
-        if (pcl_isnan (pcd_pt.x))
-          EXPECT_TRUE (pcl_isnan (pclzf_pt.x));
+        if (std::isnan (pcd_pt.x))
+          EXPECT_TRUE (std::isnan (pclzf_pt.x));
         else
           EXPECT_NEAR ( pcd_pt.x * (x-cx_new), pclzf_pt.x * fx_multiplier * (x-cx_old), 1E-4);
-        if (pcl_isnan (pcd_pt.y))
-          EXPECT_TRUE (pcl_isnan (pclzf_pt.y));
+        if (std::isnan (pcd_pt.y))
+          EXPECT_TRUE (std::isnan (pclzf_pt.y));
         else
           EXPECT_NEAR ( pcd_pt.y * (y-cy_new), pclzf_pt.y * fy_multiplier * (y-cy_old), 1E-4);
-        if (pcl_isnan (pcd_pt.z))
-          EXPECT_TRUE (pcl_isnan (pclzf_pt.z));
+        if (std::isnan (pcd_pt.z))
+          EXPECT_TRUE (std::isnan (pclzf_pt.z));
         else
           EXPECT_FLOAT_EQ (pcd_pt.z, pclzf_pt.z);
       }
@@ -522,31 +521,22 @@ int
   pclzf_dir_ = grabber_sequences + "/pclzf";
   pcd_dir_ = grabber_sequences + "/pcd";
   // Get pcd files
-  boost::filesystem::directory_iterator end_itr;
-  for (boost::filesystem::directory_iterator itr (pcd_dir_); itr != end_itr; ++itr)
+  pcl_fs::directory_iterator end_itr;
+  for (pcl_fs::directory_iterator itr (pcd_dir_); itr != end_itr; ++itr)
   {
-#if BOOST_FILESYSTEM_VERSION == 3
-    if (!is_directory (itr->status ()) && boost::algorithm::to_upper_copy (boost::filesystem::extension (itr->path ())) == ".PCD" )
-#else
-    if (!is_directory (itr->status ()) && boost::algorithm::to_upper_copy (boost::filesystem::extension (itr->leaf ())) == ".PCD" )
-#endif
+    if (!is_directory (itr->status ()) && boost::algorithm::to_upper_copy (itr->path ().extension ().string ()) == ".PCD" )
     {
-#if BOOST_FILESYSTEM_VERSION == 3
       pcd_files_.push_back (itr->path ().string ());
       std::cout << "added: " << itr->path ().string () << std::endl;
-#else
-      pcd_files_.push_back (itr->pcd_dir_ ().string ());
-      std::cout << "added: " << itr->pcd_dir_ () << std::endl;
-#endif
     }
   }
   sort (pcd_files_.begin (), pcd_files_.end ());
   // And load them
-  for (size_t i = 0; i < pcd_files_.size (); i++)
+  for (const auto &pcd_file : pcd_files_)
   {
     CloudT::Ptr cloud (new CloudT);
-    pcl::io::loadPCDFile (pcd_files_[i], *cloud); 
-    pcds_.push_back (cloud);
+    pcl::io::loadPCDFile (pcd_file, *cloud); 
+    pcds_.emplace_back(cloud);
   }
 
   testing::InitGoogleTest (&argc, argv);
