@@ -35,20 +35,21 @@
  *
  */
 
-#include <gtest/gtest.h>
+#include <pcl/test/gtest.h>
+
+#include <random>
+
 #include <pcl/search/brute_force.h>
 #include <pcl/search/kdtree.h>
+#include <pcl/search/flann_search.h>
+#include <pcl/search/kdtree_nanoflann.h>
 #include <pcl/search/organized.h>
 #include <pcl/search/octree.h>
 #include <pcl/io/pcd_io.h>
-#include <pcl/common/time.h>
-#include <boost/random/variate_generator.hpp>
-#include <boost/random/mersenne_twister.hpp>
-#include <boost/random/uniform_int.hpp>
-#include <boost/random/uniform_real.hpp>
+#include <pcl/common/point_tests.h> // for pcl::isFinite
+
 
 using namespace pcl;
-using namespace std;
 
 /** \brief if set to value other than 0 -> fine grained output */
 #define DEBUG_OUT 1
@@ -70,16 +71,16 @@ using namespace std;
 
 #if EXCESSIVE_TESTING
 /** \brief number of points used for creating unordered point clouds */
-const unsigned int unorganized_point_count = 100000;
+constexpr unsigned int unorganized_point_count = 100000;
 
 /** \brief number of search operations on ordered point clouds*/
-const unsigned int query_count = 5000;
+constexpr unsigned int query_count = 5000;
 #else
 /** \brief number of points used for creating unordered point clouds */
-const unsigned int unorganized_point_count = 1200;
+constexpr unsigned int unorganized_point_count = 1200;
 
 /** \brief number of search operations on ordered point clouds*/
-const unsigned int query_count = 100;
+constexpr unsigned int query_count = 100;
 #endif
 
 /** \brief organized point cloud*/
@@ -94,16 +95,20 @@ PointCloud<PointXYZ>::Ptr unorganized_sparse_cloud (new PointCloud<PointXYZ>);
 /** \brief unorganized point cloud*/
 PointCloud<PointXYZ>::Ptr unorganized_grid_cloud (new PointCloud<PointXYZ>);
 
+
+/** \brief random number generator*/
+std::mt19937 rng;
+
 /** \brief uniform distributed random number generator for unsigned it in range [0;10]*/
-boost::variate_generator< boost::mt19937, boost::uniform_int<unsigned> > rand_uint(boost::mt19937 (), boost::uniform_int<unsigned> (0, 10));
+std::uniform_int_distribution<unsigned> rand_uint(0, 10);
 /** \brief uniform distributed random number generator for floats in the range [0;1] */
-boost::variate_generator< boost::mt19937, boost::uniform_real<float> > rand_float(boost::mt19937 (), boost::uniform_real<float> (0, 1));
+std::uniform_real_distribution<float> rand_float (0.0f, 1.0f);
 
 /** \brief used by the *_VIEW_* tests to use only a subset of points from the point cloud*/
-std::vector<int> unorganized_input_indices;
+pcl::Indices unorganized_input_indices;
 
 /** \brief used by the *_VIEW_* tests to use only a subset of points from the point cloud*/
-std::vector<int> organized_input_indices;
+pcl::Indices organized_input_indices;
 
 /** \brief instance of brute force search method to be tested*/
 pcl::search::BruteForce<pcl::PointXYZ> brute_force;
@@ -111,6 +116,13 @@ pcl::search::BruteForce<pcl::PointXYZ> brute_force;
 /** \brief instance of KDTree search method to be tested*/
 pcl::search::KdTree<pcl::PointXYZ> KDTree;
 
+/** \brief instance of FlannSearch search method to be tested*/
+pcl::search::FlannSearch<pcl::PointXYZ> FlannSearch;
+
+#if PCL_HAS_NANOFLANN
+pcl::search::KdTreeNanoflann<pcl::PointXYZ> KDTreeNanoflann;
+#endif
+
 /** \brief instance of Octree search method to be tested*/
 pcl::search::Octree<pcl::PointXYZ> octree_search (0.1);
 
@@ -118,22 +130,22 @@ pcl::search::Octree<pcl::PointXYZ> octree_search (0.1);
 pcl::search::OrganizedNeighbor<pcl::PointXYZ> organized;
 
 /** \brief list of search methods for unorganized search test*/
-vector<search::Search<PointXYZ>* > unorganized_search_methods;
+std::vector<search::Search<PointXYZ>* > unorganized_search_methods;
 
 /** \brief list of search methods for organized search test*/
-vector<search::Search<PointXYZ>* > organized_search_methods;
+std::vector<search::Search<PointXYZ>* > organized_search_methods;
 
 /** \brief lists of indices to be used as query points for various search methods and different cloud types*/
-vector<int> unorganized_dense_cloud_query_indices;
-vector<int> unorganized_sparse_cloud_query_indices;
-vector<int> organized_sparse_query_indices;
+pcl::Indices unorganized_dense_cloud_query_indices;
+pcl::Indices unorganized_sparse_cloud_query_indices;
+pcl::Indices organized_sparse_query_indices;
 
-/** \briet test whether the result of a search containes unique point ids or not
+/** \briet test whether the result of a search contains unique point ids or not
   * @param indices resulting indices from a search
   * @param name name of the search method that returned these distances
   * @return true if indices are unique, false otherwise
   */
-bool testUniqueness (const vector<int>& indices, const string& name)
+bool testUniqueness (const pcl::Indices& indices, const std::string& name)
 {
   bool uniqueness = true;
   for (unsigned idx1 = 1; idx1 < indices.size () && uniqueness; ++idx1)
@@ -161,10 +173,10 @@ bool testUniqueness (const vector<int>& indices, const string& name)
   * \param name name of the search method that returned these distances
   * \return true if distances in weak ascending order, false otherwise
   */
-bool testOrder (const vector<float>& distances, const string& name)
+bool testOrder (const std::vector<float>& distances, const std::string& name)
 {
   bool ordered = true;
-  for (unsigned idx1 = 1; idx1 < distances.size (); ++idx1)
+  for (std::size_t idx1 = 1; idx1 < distances.size (); ++idx1)
   {
     if (distances [idx1-1] > distances [idx1])
     {
@@ -188,29 +200,29 @@ bool testOrder (const vector<float>& distances, const string& name)
  * @return true if result is valid, false otherwise
  */
 template<typename PointT> bool
-testResultValidity (const typename PointCloud<PointT>::ConstPtr point_cloud, const vector<bool>& indices_mask, const vector<bool>& nan_mask, const vector<int>& indices, const vector<int>& /*input_indices*/, const string& name)
+testResultValidity (const typename PointCloud<PointT>::ConstPtr point_cloud, const std::vector<bool>& indices_mask, const std::vector<bool>& nan_mask, const pcl::Indices& indices, const pcl::Indices& /*input_indices*/, const std::string& name)
 {
   bool validness = true;
-  for (vector<int>::const_iterator iIt = indices.begin (); iIt != indices.end (); ++iIt)
+  for (const auto &index : indices)
   {
-    if (!indices_mask [*iIt])
+    if (!indices_mask [index])
     {
 #if DEBUG_OUT
-      cerr << name << ": result contains an invalid point: " << *iIt << " not in indices list.\n";
+      std::cerr << name << ": result contains an invalid point: " << index << " not in indices list.\n";
       
 //      for (vector<int>::const_iterator iIt2 = input_indices.begin (); iIt2 != input_indices.end (); ++iIt2)
-//        cout << *iIt2 << "  ";
-//      cout << endl;
+//        std::cout << *iIt2 << "  ";
+//      std::cout << std::endl;
 #endif
       validness = false;
       break;
     }
-    else if (!nan_mask [*iIt])
+    if (!nan_mask [index])
     {
 #if DEBUG_OUT
-      cerr << name << ": result contains an invalid point: " << *iIt << " = NaN (" << point_cloud->points [*iIt].x << " , " 
-                                                                                   << point_cloud->points [*iIt].y << " , " 
-                                                                                   << point_cloud->points [*iIt].z << ")\n";
+      std::cerr << name << ": result contains an invalid point: " << index << " = NaN (" << point_cloud->points [index].x << " , " 
+                                                                                    << point_cloud->points [index].y << " , " 
+                                                                                    << point_cloud->points [index].z << ")\n";
 #endif
       validness = false;
       break;
@@ -230,38 +242,38 @@ testResultValidity (const typename PointCloud<PointT>::ConstPtr point_cloud, con
   * \param eps threshold for comparing the distances
   * \return true if both sets are the same, false otherwise
   */
-bool compareResults (const std::vector<int>& indices1, const::vector<float>& distances1, const std::string& name1,
-                     const std::vector<int>& indices2, const::vector<float>& distances2, const std::string& name2, float eps)
+bool compareResults (const pcl::Indices& indices1, const std::vector<float>& distances1, const std::string& name1,
+                     const pcl::Indices& indices2, const std::vector<float>& distances2, const std::string& name2, float eps)
 {
   bool equal = true;
   if (indices1.size () != indices2.size ())
   {
 #if DEBUG_OUT
-    cerr << "size of results between " << name1 << " search and " << name2 << " search do not match " <<indices1.size () << " vs. " << indices2.size () << endl;
+    std::cerr << "size of results between " << name1 << " search and " << name2 << " search do not match " <<indices1.size () << " vs. " << indices2.size () << std::endl;
 //    for (unsigned idx = 0; idx < std::min (indices1.size (), indices2.size ()); ++idx)
 //    {
-//      cout << idx <<".\t" << indices1[idx] << "\t(" << distances1[idx] << "),\t" << indices2[idx] << "\t(" << distances2[idx] << ")\n";
+//      std::cout << idx <<".\t" << indices1[idx] << "\t(" << distances1[idx] << "),\t" << indices2[idx] << "\t(" << distances2[idx] << ")\n";
 //    }    
 //    for (unsigned idx = std::min (indices1.size (), indices2.size ()); idx < std::max (indices1.size (), indices2.size ()); ++idx)
 //    {
 //      if (idx >= indices1.size ())
-//        cout << idx <<".\t     \t      ,\t" << indices2[idx] << "\t(" << distances2[idx] << ")\n";
+//        std::cout << idx <<".\t     \t      ,\t" << indices2[idx] << "\t(" << distances2[idx] << ")\n";
 //      else
-//        cout << idx <<".\t" << indices1[idx] << "\t(" << distances1[idx] << ")\n";
+//        std::cout << idx <<".\t" << indices1[idx] << "\t(" << distances1[idx] << ")\n";
 //    }
 #endif
     equal = false;
   }
   else
   {
-    for (unsigned idx = 0; idx < indices1.size (); ++idx)
+    for (std::size_t idx = 0; idx < indices1.size (); ++idx)
     {
-      if (indices1[idx] != indices2[idx] && fabs (distances1[idx] - distances2[idx]) > eps)
+      if (indices1[idx] != indices2[idx] && std::abs (distances1[idx] - distances2[idx]) > eps)
       {
 #if DEBUG_OUT
-        cerr << "results between " << name1 << " search and " << name2 << " search do not match: " << idx << " nearest neighbor: "
+        std::cerr << "results between " << name1 << " search and " << name2 << " search do not match: " << idx << " nearest neighbor: "
                 << indices1[idx] << " with distance: " << distances1[idx] << " vs. "
-                << indices2[idx] << " with distance: " << distances2[idx] << endl;
+                << indices2[idx] << " with distance: " << distances2[idx] << std::endl;
 #endif
         equal = false;
         break;
@@ -278,66 +290,74 @@ bool compareResults (const std::vector<int>& indices1, const::vector<float>& dis
   * \param input_indices indices defining a subset of the point cloud.
   */
 template<typename PointT> void
-testKNNSearch (typename PointCloud<PointT>::ConstPtr point_cloud, vector<search::Search<PointT>*> search_methods,
-                const vector<int>& query_indices, const vector<int>& input_indices = vector<int> () )
+testKNNSearch (typename PointCloud<PointT>::ConstPtr point_cloud, std::vector<search::Search<PointT>*> search_methods,
+                const pcl::Indices& query_indices, const pcl::Indices& input_indices = pcl::Indices () )
 {
-  vector< vector<int> >indices (search_methods.size ());
-  vector< vector<float> >distances (search_methods.size ());
-  vector<bool> passed (search_methods.size (), true);
+  std::vector< pcl::Indices >indices (search_methods.size ());
+  std::vector< std::vector<float> >distances (search_methods.size ());
+  std::vector<bool> passed (search_methods.size (), true);
   
-  vector<bool> indices_mask (point_cloud->size (), true);
-  vector<bool> nan_mask (point_cloud->size (), true);
+  std::vector<bool> indices_mask (point_cloud->size (), true);
+  std::vector<bool> nan_mask (point_cloud->size (), true);
   
-  if (input_indices.size () != 0)
+  if (!input_indices.empty ())
   {
     indices_mask.assign (point_cloud->size (), false);
-    for (vector<int>::const_iterator iIt = input_indices.begin (); iIt != input_indices.end (); ++iIt)
-      indices_mask [*iIt] = true;
+    for (const auto &input_index : input_indices)
+      indices_mask [input_index] = true;
   }
   
   // remove also Nans
-  #pragma omp parallel for
-  for (int pIdx = 0; pIdx < int (point_cloud->size ()); ++pIdx)
+  #pragma omp parallel for \
+    shared(nan_mask, point_cloud) \
+    default(none)
+  for (int pIdx = 0; pIdx < static_cast<int>(point_cloud->size ()); ++pIdx)
   {
     if (!isFinite (point_cloud->points [pIdx]))
       nan_mask [pIdx] = false;
   }
   
-  boost::shared_ptr<vector<int> > input_indices_;
-  if (input_indices.size ())
-    input_indices_.reset (new vector<int> (input_indices));
+  pcl::IndicesPtr input_indices_;
+  if (!input_indices.empty ())
+    input_indices_.reset (new pcl::Indices (input_indices));
   
-  #pragma omp parallel for
-  for (int sIdx = 0; sIdx < int (search_methods.size ()); ++sIdx)
+  #pragma omp parallel for \
+    shared(input_indices, input_indices_, point_cloud, search_methods) \
+    default(none)
+  for (int sIdx = 0; sIdx < static_cast<int>(search_methods.size ()); ++sIdx)
     search_methods [sIdx]->setInputCloud (point_cloud, input_indices_);
 
   // test knn values from 1, 8, 64, 512
   for (unsigned knn = 1; knn <= 512; knn <<= 3)
   {
     // find nn for each point in the cloud
-    for (vector<int>::const_iterator qIt = query_indices.begin (); qIt != query_indices.end (); ++qIt)
+    for (const auto &query_index : query_indices)
     {
-      #pragma omp parallel for
-      for (int sIdx = 0; sIdx < int (search_methods.size ()); ++sIdx)
+      #pragma omp parallel for \
+        shared(indices, input_indices, indices_mask, distances, knn, nan_mask, passed, point_cloud, query_index, search_methods) \
+        default(none)
+      for (int sIdx = 0; sIdx < static_cast<int>(search_methods.size ()); ++sIdx)
       {
-        search_methods [sIdx]->nearestKSearch (point_cloud->points[*qIt], knn, indices [sIdx], distances [sIdx]);
+        search_methods [sIdx]->nearestKSearch ((*point_cloud)[query_index], knn, indices [sIdx], distances [sIdx]);
         passed [sIdx] = passed [sIdx] && testUniqueness (indices [sIdx], search_methods [sIdx]->getName ());
         passed [sIdx] = passed [sIdx] && testOrder (distances [sIdx], search_methods [sIdx]->getName ());
         passed [sIdx] = passed [sIdx] && testResultValidity<PointT>(point_cloud, indices_mask, nan_mask, indices [sIdx], input_indices, search_methods [sIdx]->getName ());
       }
       
       // compare results to each other
-      #pragma omp parallel for
-      for (int sIdx = 1; sIdx < int (search_methods.size ()); ++sIdx)
+      #pragma omp parallel for \
+        shared(distances, indices, passed, search_methods) \
+        default(none)
+      for (int sIdx = 1; sIdx < static_cast<int>(search_methods.size ()); ++sIdx)
       {
         passed [sIdx] = passed [sIdx] && compareResults (indices [0],    distances [0],    search_methods [0]->getName (),
                                                          indices [sIdx], distances [sIdx], search_methods [sIdx]->getName (), 1e-6f);
       }
     }
   }
-  for (size_t sIdx = 0; sIdx < search_methods.size (); ++sIdx)
+  for (std::size_t sIdx = 0; sIdx < search_methods.size (); ++sIdx)
   {
-    cout << search_methods [sIdx]->getName () << ": " << (passed[sIdx]?"passed":"failed") << endl;
+    std::cout << search_methods [sIdx]->getName () << ": " << (passed[sIdx]?"passed":"failed") << std::endl;
     EXPECT_TRUE (passed [sIdx]);
   }
 }
@@ -349,66 +369,84 @@ testKNNSearch (typename PointCloud<PointT>::ConstPtr point_cloud, vector<search:
   * \param input_indices indices defining a subset of the point cloud.
   */
 template<typename PointT> void
-testRadiusSearch (typename PointCloud<PointT>::ConstPtr point_cloud, vector<search::Search<PointT>*> search_methods, 
-                   const vector<int>& query_indices, const vector<int>& input_indices = vector<int> ())
+testRadiusSearch (typename PointCloud<PointT>::ConstPtr point_cloud, std::vector<search::Search<PointT>*> search_methods, 
+                   const pcl::Indices& query_indices, const pcl::Indices& input_indices = pcl::Indices   ())
 {
-  vector< vector<int> >indices (search_methods.size ());
-  vector< vector<float> >distances (search_methods.size ());
-  vector <bool> passed (search_methods.size (), true);
-  vector<bool> indices_mask (point_cloud->size (), true);
-  vector<bool> nan_mask (point_cloud->size (), true);
+  std::vector< pcl::Indices >indices (search_methods.size ());
+  std::vector< std::vector<float> >distances (search_methods.size ());
+  std::vector<bool> passed (search_methods.size (), true);
+  std::vector<bool> indices_mask (point_cloud->size (), true);
+  std::vector<bool> nan_mask (point_cloud->size (), true);
   
-  if (input_indices.size () != 0)
+  if (!input_indices.empty ())
   {
     indices_mask.assign (point_cloud->size (), false);
-    for (vector<int>::const_iterator iIt = input_indices.begin (); iIt != input_indices.end (); ++iIt)
-      indices_mask [*iIt] = true;
+    for (const auto &input_index : input_indices)
+      indices_mask [input_index] = true;
   }
   
   // remove also Nans
-  #pragma omp parallel for
-  for (int pIdx = 0; pIdx < int (point_cloud->size ()); ++pIdx)
+  #pragma omp parallel for \
+    default(none) \
+    shared(nan_mask, point_cloud)
+  for (int pIdx = 0; pIdx < static_cast<int>(point_cloud->size ()); ++pIdx)
   {
     if (!isFinite (point_cloud->points [pIdx]))
       nan_mask [pIdx] = false;
   }
   
-  boost::shared_ptr<vector<int> > input_indices_;
-  if (input_indices.size ())
-    input_indices_.reset (new vector<int> (input_indices));
+  pcl::IndicesPtr input_indices_;
+  if (!input_indices.empty ())
+    input_indices_.reset (new pcl::Indices (input_indices));
   
-  #pragma omp parallel for
-  for (int sIdx = 0; sIdx < int (search_methods.size ()); ++sIdx)
+  #pragma omp parallel for \
+    default(none) \
+    shared(input_indices_, point_cloud, search_methods)
+  for (int sIdx = 0; sIdx < static_cast<int>(search_methods.size ()); ++sIdx)
     search_methods [sIdx]->setInputCloud (point_cloud, input_indices_);
 
   // test radii 0.01, 0.02, 0.04, 0.08
   for (float radius = 0.01f; radius < 0.1f; radius *= 2.0f)
   {
-    //cout << radius << endl;
+    //std::cout << radius << std::endl;
     // find nn for each point in the cloud
-    for (vector<int>::const_iterator qIt = query_indices.begin (); qIt != query_indices.end (); ++qIt)
+    for (const auto &query_index : query_indices)
     {
-      #pragma omp parallel for
+      #pragma omp parallel for \
+        default(none) \
+        shared(distances, indices, indices_mask, input_indices, nan_mask, passed, point_cloud, radius, query_index, search_methods)
       for (int sIdx = 0; sIdx < static_cast<int> (search_methods.size ()); ++sIdx)
       {
-        search_methods [sIdx]->radiusSearch (point_cloud->points[*qIt], radius, indices [sIdx], distances [sIdx], 0);
+        search_methods [sIdx]->radiusSearch ((*point_cloud)[query_index], radius, indices [sIdx], distances [sIdx], 0);
         passed [sIdx] = passed [sIdx] && testUniqueness (indices [sIdx], search_methods [sIdx]->getName ());
         passed [sIdx] = passed [sIdx] && testOrder (distances [sIdx], search_methods [sIdx]->getName ());
         passed [sIdx] = passed [sIdx] && testResultValidity<PointT>(point_cloud, indices_mask, nan_mask, indices [sIdx], input_indices, search_methods [sIdx]->getName ());
       }
       
       // compare results to each other
-      #pragma omp parallel for
+      #pragma omp parallel for \
+        default(none) \
+        shared(distances, indices, passed, search_methods, radius)
       for (int sIdx = 1; sIdx < static_cast<int> (search_methods.size ()); ++sIdx)
       {
-        passed [sIdx] = passed [sIdx] && compareResults (indices [0],    distances [0],    search_methods [0]->getName (),
-                                                         indices [sIdx], distances [sIdx], search_methods [sIdx]->getName (), 1e-6f);
+        const bool same_results = compareResults (indices [0],    distances [0],    search_methods [0]->getName (),
+                                                  indices [sIdx], distances [sIdx], search_methods [sIdx]->getName (), 1e-6f);
+        if (!same_results) {
+          if ((((indices [0   ].size()+1)==indices [sIdx].size()) && std::abs(*distances [sIdx].crbegin()-radius*radius)<1e-6) ||
+              (((indices [sIdx].size()+1)==indices [0   ].size()) && std::abs(*distances [0   ].crbegin()-radius*radius)<1e-6)) {
+            // One result list has one entry more than the other, and this additional entry is very close to the radius boundary.
+            // Because of numerical inaccuracies, points very close to the boundary may be counted as inside or outside depending
+            // on the search method. The two result lists will still be considered the same in this case.
+          } else {
+            passed [sIdx] = false;
+          }
+        }
       }
     }
   }
-  for (unsigned sIdx = 0; sIdx < search_methods.size (); ++sIdx)
+  for (std::size_t sIdx = 0; sIdx < search_methods.size (); ++sIdx)
   {
-    cout << search_methods [sIdx]->getName () << ": " << (passed[sIdx]?"passed":"failed") << endl;
+    std::cout << search_methods [sIdx]->getName () << ": " << (passed[sIdx]?"passed":"failed") << std::endl;
     EXPECT_TRUE (passed [sIdx]);
   }
 }
@@ -456,7 +494,7 @@ TEST (PCL, unorganized_dense_cloud_Complete_Radius)
 // Test search on unorganized point clouds in a grid
 TEST (PCL, unorganized_grid_cloud_Complete_Radius)
 {
-  vector<int> query_indices;
+  pcl::Indices query_indices;
   query_indices.reserve (query_count);
   
   unsigned skip = static_cast<unsigned> (unorganized_grid_cloud->size ()) / query_count;
@@ -524,7 +562,7 @@ TEST (PCL, Organized_Sparse_View_Radius)
   * \param cloud input cloud required to check for nans and to get number of points
   * \param[in] query_count maximum number of query points
   */
-void createQueryIndices (std::vector<int>& query_indices, PointCloud<PointXYZ>::ConstPtr point_cloud, unsigned query_count)
+void createQueryIndices (pcl::Indices& query_indices, PointCloud<PointXYZ>::ConstPtr point_cloud, unsigned query_count)
 {
   query_indices.clear ();
   query_indices.reserve (query_count);
@@ -539,19 +577,19 @@ void createQueryIndices (std::vector<int>& query_indices, PointCloud<PointXYZ>::
   * \param indices 
   * \param max_index highest accented index usually given by cloud->size () - 1
   */
-void createIndices (std::vector<int>& indices, unsigned max_index)
+void createIndices (pcl::Indices& indices, unsigned max_index)
 {
   // ~10% of the input cloud
   for (unsigned idx = 0; idx <= max_index; ++idx)
-    if (rand_uint () == 0)
+    if (rand_uint (rng) == 0)
       indices.push_back (idx);
    
-  boost::variate_generator< boost::mt19937, boost::uniform_int<> > rand_indices(boost::mt19937 (), boost::uniform_int<> (0, static_cast<int> (indices.size ()) - 1));
+  std::uniform_int_distribution<> rand_indices(0, indices.size () - 1);
   // shuffle indices -> not ascending index list
   for (unsigned idx = 0; idx < max_index; ++idx)
   {
-    unsigned idx1 = rand_indices ();
-    unsigned idx2 = rand_indices ();
+    const unsigned idx1 = rand_indices (rng);
+    const unsigned idx2 = rand_indices (rng);
 
     std::swap (indices[idx1], indices[idx2]);
   }
@@ -569,6 +607,9 @@ main (int argc, char** argv)
 
   pcl::io::loadPCDFile (argv [1], *organized_sparse_cloud);
   
+  const unsigned int seed = time (nullptr);
+  srand (seed);
+
   // create unorganized cloud
   unorganized_dense_cloud->resize (unorganized_point_count);
   unorganized_dense_cloud->height = 1;
@@ -583,13 +624,13 @@ main (int argc, char** argv)
   PointXYZ point;
   for (unsigned pIdx = 0; pIdx < unorganized_point_count; ++pIdx)
   {
-    point.x = rand_float ();
-    point.y = rand_float ();
-    point.z = rand_float ();
+    point.x = rand_float (rng);
+    point.y = rand_float (rng);
+    point.z = rand_float (rng);
 
     unorganized_dense_cloud->points [pIdx] = point;
     
-    if (rand_uint () == 0)
+    if (rand_uint (rng) == 0)
       unorganized_sparse_cloud->points [pIdx].x = unorganized_sparse_cloud->points [pIdx].y = unorganized_sparse_cloud->points [pIdx].z = std::numeric_limits<float>::quiet_NaN ();
     else
       unorganized_sparse_cloud->points [pIdx] = point;
@@ -620,15 +661,26 @@ main (int argc, char** argv)
   
   brute_force.setSortedResults (true);
   KDTree.setSortedResults (true);
+#if PCL_HAS_NANOFLANN
+  KDTreeNanoflann.setSortedResults (true);
+#endif
   octree_search.setSortedResults (true);
   organized.setSortedResults (true);
   
   unorganized_search_methods.push_back (&brute_force);
   unorganized_search_methods.push_back (&KDTree);
+  unorganized_search_methods.push_back (&FlannSearch);
+#if PCL_HAS_NANOFLANN
+  unorganized_search_methods.push_back (&KDTreeNanoflann);
+#endif
   unorganized_search_methods.push_back (&octree_search);
   
   organized_search_methods.push_back (&brute_force);
   organized_search_methods.push_back (&KDTree);
+  organized_search_methods.push_back (&FlannSearch);
+#if PCL_HAS_NANOFLANN
+  organized_search_methods.push_back (&KDTreeNanoflann);
+#endif
   organized_search_methods.push_back (&octree_search);
   organized_search_methods.push_back (&organized);
   
