@@ -36,138 +36,49 @@
  *
  */
 
-#include <pcl/common/common_headers.h>
+#include <thread>
+
+#include <pcl/common/time.h> // for DO_EVERY
 #include <pcl/visualization/common/common.h>
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-#include <pcl/visualization/interactor.h>
-#else
 #include <vtkRenderWindowInteractor.h>
-#endif
 #include <pcl/visualization/histogram_visualizer.h>
-#include <pcl/visualization/boost.h>
 
 #include <vtkVersion.h>
-#include <vtkXYPlotActor.h>
 #include <vtkDoubleArray.h>
 #include <vtkTextProperty.h>
-#include <vtkRenderWindow.h>
 #include <vtkRenderer.h>
 #include <vtkDataObject.h>
 #include <vtkProperty2D.h>
 #include <vtkFieldData.h>
 
+using namespace std::chrono_literals;
+
 //////////////////////////////////////////////////////////////////////////////////////////////
 pcl::visualization::PCLHistogramVisualizer::PCLHistogramVisualizer () : 
-  wins_ (),
   exit_main_loop_timer_callback_ (vtkSmartPointer<ExitMainLoopTimerCallback>::New ()), 
-  exit_callback_ (vtkSmartPointer<ExitCallback>::New ()), 
-  stopped_ ()
+  exit_callback_ (vtkSmartPointer<ExitCallback>::New ())
 {
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-  resetStoppedFlag ();
-#endif
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////
 /** \brief Spin once method. Calls the interactor and updates the screen once. */
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-//////////////////////////////////////////////////////////////////////////////////////////////
-/** \brief Spin once method. Calls the interactor and updates the screen once. */
-void
-pcl::visualization::PCLHistogramVisualizer::spinOnce (int time, bool force_redraw)
-{
-  resetStoppedFlag ();
-
-  if (time <= 0)
-    time = 1;
-  
-  if (force_redraw)
-  {
-    for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
-    {
-      (*am_it).second.interactor_->Render ();
-      exit_main_loop_timer_callback_->right_timer_id = (*am_it).second.interactor_->CreateRepeatingTimer (time);
-
-      exit_main_loop_timer_callback_->interact = (*am_it).second.interactor_;
-
-      (*am_it).second.interactor_->Start ();
-      (*am_it).second.interactor_->DestroyTimer (exit_main_loop_timer_callback_->right_timer_id);
-    }
-    return;
-  }
-  
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
-  {
-    DO_EVERY(1.0/(*am_it).second.interactor_->GetDesiredUpdateRate (),
-      (*am_it).second.interactor_->Render ();
-      exit_main_loop_timer_callback_->right_timer_id = (*am_it).second.interactor_->CreateRepeatingTimer (time);
-
-      exit_main_loop_timer_callback_->interact = (*am_it).second.interactor_;
-
-      (*am_it).second.interactor_->Start ();
-      (*am_it).second.interactor_->DestroyTimer (exit_main_loop_timer_callback_->right_timer_id);
-    );
-  }
-}
-#else
 void
 pcl::visualization::PCLHistogramVisualizer::spinOnce (int time)
 {
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
+  for (auto &win : wins_)
   {
-    DO_EVERY(1.0/(*am_it).second.interactor_->GetDesiredUpdateRate (),
-      (*am_it).second.interactor_->Render ();
-      exit_main_loop_timer_callback_->right_timer_id = (*am_it).second.interactor_->CreateRepeatingTimer (time);
+    DO_EVERY(1.0/win.second.interactor_->GetDesiredUpdateRate (),
+      win.second.interactor_->Render ();
+      exit_main_loop_timer_callback_->right_timer_id = win.second.interactor_->CreateRepeatingTimer (time);
 
       // Set the correct interactor for callbacks
-      exit_main_loop_timer_callback_->interact = (*am_it).second.interactor_;
-      (*am_it).second.interactor_->Start ();
-      (*am_it).second.interactor_->DestroyTimer (exit_main_loop_timer_callback_->right_timer_id);
+      exit_main_loop_timer_callback_->interact = win.second.interactor_;
+      win.second.interactor_->Start ();
+      win.second.interactor_->DestroyTimer (exit_main_loop_timer_callback_->right_timer_id);
     );
   }
 }
-#endif
 
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-//////////////////////////////////////////////////////////////////////////////////////////////
-void
-pcl::visualization::PCLHistogramVisualizer::spin ()
-{
-  resetStoppedFlag ();
-  do
-  {
-    spinOnce ();
-    for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
-    {
-      if ((*am_it).second.interactor_->stopped)
-        return;
-    }
-    boost::this_thread::sleep (boost::posix_time::seconds (1));
-  }
-  while (true);
-}
-////////////////////////////////////////////////////////////////////////////////////////////
-bool 
-pcl::visualization::PCLHistogramVisualizer::wasStopped ()
-{
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
-  {
-    // If any of the interactors was stopped, return true (stop everything else)
-    if ((*am_it).second.interactor_->stopped)
-      return (true);
-  }
-  return (false);
-}
-
-////////////////////////////////////////////////////////////////////////////////////////////
-void 
-pcl::visualization::PCLHistogramVisualizer::resetStoppedFlag () 
-{ 
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
-    (*am_it).second.interactor_->stopped = false; 
-}
-
-#else
 //////////////////////////////////////////////////////////////////////////////////////////////
 void
 pcl::visualization::PCLHistogramVisualizer::spin ()
@@ -178,11 +89,10 @@ pcl::visualization::PCLHistogramVisualizer::spin ()
     spinOnce ();
     if (stopped_)
       break;
-    boost::this_thread::sleep (boost::posix_time::seconds (1));
+    std::this_thread::sleep_for(1s);
   }
   while (true);
 }
-#endif
 
 //////////////////////////////////////////////////////////////////////////////////////////////
 void 
@@ -208,10 +118,10 @@ pcl::visualization::PCLHistogramVisualizer::setBackgroundColor (const double &r,
     ++i;
   }
   */
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
+  for (auto &win : wins_)
   {
-    (*am_it).second.ren_->SetBackground (r, g, b);
-    (*am_it).second.ren_->Render ();
+    win.second.ren_->SetBackground (r, g, b);
+    win.second.ren_->Render ();
   }
 }
 
@@ -219,10 +129,10 @@ pcl::visualization::PCLHistogramVisualizer::setBackgroundColor (const double &r,
 void 
 pcl::visualization::PCLHistogramVisualizer::setGlobalYRange (float minp, float maxp)
 {
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
+  for (auto &win : wins_)
   {
-    (*am_it).second.xy_plot_->SetYRange (minp, maxp);
-    (*am_it).second.xy_plot_->Modified ();
+    win.second.xy_plot_->SetYRange (minp, maxp);
+    win.second.xy_plot_->Modified ();
   }
 }
 
@@ -231,15 +141,15 @@ void
 pcl::visualization::PCLHistogramVisualizer::updateWindowPositions ()
 {
   int posx = 0, posy = 0;
-  for (RenWinInteractMap::iterator am_it = wins_.begin (); am_it != wins_.end (); ++am_it)
+  for (auto &win : wins_)
   {
     // Get the screen size
-    int *scr_size = (*am_it).second.win_->GetScreenSize ();
-    int *win_size = (*am_it).second.win_->GetActualSize ();
+    int *scr_size = win.second.win_->GetScreenSize ();
+    int *win_size = win.second.win_->GetActualSize ();
 
     // Update the position of the current window
-    (*am_it).second.win_->SetPosition (posx, posy);
-    (*am_it).second.win_->Modified ();
+    win.second.win_->SetPosition (posx, posy);
+    win.second.win_->Modified ();
     // If there is space on Y, go on Y first
     if ((posy + win_size[1]) <= scr_size[1]) 
       posy += win_size[1];
@@ -261,12 +171,7 @@ pcl::visualization::PCLHistogramVisualizer::reCreateActor (
     const vtkSmartPointer<vtkDoubleArray> &xy_array, RenWinInteract* renwinupd, const int hsize)
 {
   renwinupd->ren_->RemoveActor2D (renwinupd->xy_plot_);
-#if VTK_MAJOR_VERSION < 6
-  renwinupd->xy_plot_->RemoveAllInputs ();
-#else
   renwinupd->xy_plot_->RemoveAllDataSetInputConnections ();
-#endif
-
   
   double min_max[2];
   xy_array->GetRange (min_max, 1);
@@ -279,7 +184,7 @@ pcl::visualization::PCLHistogramVisualizer::reCreateActor (
   field_data->SetFieldData (field_values);
 
   renwinupd->xy_plot_->AddDataObjectInput (field_data);
-  renwinupd->ren_->AddActor2D (renwinupd->xy_plot_);
+  renwinupd->ren_->AddViewProp (renwinupd->xy_plot_);
   
   renwinupd->xy_plot_->SetYTitle (""); renwinupd->xy_plot_->SetXTitle ("");
   renwinupd->xy_plot_->SetYRange (min_max[0], min_max[1]); 
@@ -343,7 +248,7 @@ pcl::visualization::PCLHistogramVisualizer::createActor (
   renwinint.xy_plot_->SetWidth (1); renwinint.xy_plot_->SetHeight (1);
 
   // Create the new window with its interactor and renderer
-  renwinint.ren_->AddActor2D (renwinint.xy_plot_);
+  renwinint.ren_->AddViewProp (renwinint.xy_plot_);
   renwinint.ren_->SetBackground (1, 1, 1);
   renwinint.win_->SetWindowName (id.c_str ());
   renwinint.win_->AddRenderer (renwinint.ren_);
@@ -355,11 +260,7 @@ pcl::visualization::PCLHistogramVisualizer::createActor (
   style_->Initialize ();
   renwinint.style_ = style_;
   renwinint.style_->UseTimersOn ();
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-  renwinint.interactor_ = vtkSmartPointer<PCLVisualizerInteractor>::New ();
-#else
   renwinint.interactor_ = vtkSmartPointer<vtkRenderWindowInteractor>::New ();
-#endif
   renwinint.interactor_->SetRenderWindow (renwinint.win_);
   renwinint.interactor_->SetInteractorStyle (renwinint.style_);
   // Initialize and create timer
@@ -368,9 +269,7 @@ pcl::visualization::PCLHistogramVisualizer::createActor (
 
   exit_main_loop_timer_callback_->right_timer_id = -1;
   renwinint.interactor_->AddObserver (vtkCommand::TimerEvent, exit_main_loop_timer_callback_);
-#if !((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
   exit_callback_->his = this;
-#endif
   renwinint.interactor_->AddObserver (vtkCommand::ExitEvent, exit_callback_);
 }
 
@@ -384,11 +283,11 @@ pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
   int field_idx = pcl::getFieldIndex (cloud, field_name);
   if (field_idx == -1)
   {
-    PCL_ERROR ("[addFeatureHistogram] Invalid field (%s) given!", field_name.c_str ());
+    PCL_ERROR ("[addFeatureHistogram] Invalid field (%s) given!\n", field_name.c_str ());
     return (false);
   }
 
-  RenWinInteractMap::iterator am_it = wins_.find (id);
+  auto am_it = wins_.find (id);
   if (am_it != wins_.end ())
   {
     PCL_WARN ("[addFeatureHistogram] A window with id <%s> already exists! Please choose a different id and retry.\n", id.c_str ());
@@ -401,7 +300,7 @@ pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
 
   // Parse the cloud data and store it in the array
   double xy[2];
-  for (unsigned int d = 0; d < cloud.fields[field_idx].count; ++d)
+  for (uindex_t d = 0; d < cloud.fields[field_idx].count; ++d)
   {
     xy[0] = d;
     float data;
@@ -415,9 +314,6 @@ pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
 
   // Save the pointer/ID pair to the global window map
   wins_[id] = renwinint;
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-  resetStoppedFlag ();
-#endif
   return (true);
 }
 
@@ -426,10 +322,10 @@ bool
 pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
     const pcl::PCLPointCloud2 &cloud,
     const std::string &field_name, 
-    const int index,
+    const pcl::index_t index,
     const std::string &id, int win_width, int win_height)
 {
-  if (index < 0 || index >= static_cast<int> (cloud.width * cloud.height))
+  if (index < 0 || index >= static_cast<pcl::index_t> (cloud.width * cloud.height))
   {
     PCL_ERROR ("[addFeatureHistogram] Invalid point index (%d) given!\n", index);
     return (false);
@@ -443,7 +339,7 @@ pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
     return (false);
   }
 
-  RenWinInteractMap::iterator am_it = wins_.find (id);
+  auto am_it = wins_.find (id);
   if (am_it != wins_.end ())
   {
     PCL_ERROR ("[addFeatureHistogram] A window with id <%s> already exists! Please choose a different id and retry.\n", id.c_str ());
@@ -456,12 +352,12 @@ pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
 
   // Compute the total size of the fields
   unsigned int fsize = 0;
-  for (size_t i = 0; i < cloud.fields.size (); ++i)
-    fsize += cloud.fields[i].count * pcl::getFieldSize (cloud.fields[i].datatype);
+  for (const auto &field : cloud.fields)
+    fsize += field.count * pcl::getFieldSize (field.datatype);
 
   // Parse the cloud data and store it in the array
   double xy[2];
-  for (unsigned int d = 0; d < cloud.fields[field_idx].count; ++d)
+  for (uindex_t d = 0; d < cloud.fields[field_idx].count; ++d)
   {
     xy[0] = d;
     float data;
@@ -475,9 +371,6 @@ pcl::visualization::PCLHistogramVisualizer::addFeatureHistogram (
 
   // Save the pointer/ID pair to the global window map
   wins_[id] = renwinint;
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-  resetStoppedFlag ();
-#endif
   return (true);
 }
 
@@ -487,7 +380,7 @@ pcl::visualization::PCLHistogramVisualizer::updateFeatureHistogram (
     const pcl::PCLPointCloud2 &cloud, const std::string &field_name,
     const std::string &id)
 {
-  RenWinInteractMap::iterator am_it = wins_.find (id);
+  auto am_it = wins_.find (id);
   if (am_it == wins_.end ())
   {
     PCL_WARN ("[updateFeatureHistogram] A window with id <%s> does not exists!.\n", id.c_str ());
@@ -508,7 +401,7 @@ pcl::visualization::PCLHistogramVisualizer::updateFeatureHistogram (
 
   // Parse the cloud data and store it in the array
   double xy[2];
-  for (unsigned int d = 0; d < cloud.fields[field_idx].count; ++d)
+  for (uindex_t d = 0; d < cloud.fields[field_idx].count; ++d)
   {
     xy[0] = d;
     float data;
@@ -525,15 +418,15 @@ bool
 pcl::visualization::PCLHistogramVisualizer::updateFeatureHistogram (
     const pcl::PCLPointCloud2 &cloud,
     const std::string &field_name, 
-    const int index,
+    const pcl::index_t index,
     const std::string &id)
 {
-  if (index < 0 || index >= static_cast<int> (cloud.width * cloud.height))
+  if (index < 0 || index >= static_cast<pcl::index_t> (cloud.width * cloud.height))
   {
     PCL_ERROR ("[updateFeatureHistogram] Invalid point index (%d) given!\n", index);
     return (false);
   }
-  RenWinInteractMap::iterator am_it = wins_.find (id);
+  auto am_it = wins_.find (id);
   if (am_it == wins_.end ())
   {
     PCL_WARN ("[updateFeatureHistogram] A window with id <%s> does not exists!.\n", id.c_str ());
@@ -554,12 +447,12 @@ pcl::visualization::PCLHistogramVisualizer::updateFeatureHistogram (
   
   // Compute the total size of the fields
   unsigned int fsize = 0;
-  for (size_t i = 0; i < cloud.fields.size (); ++i)
-    fsize += cloud.fields[i].count * pcl::getFieldSize (cloud.fields[i].datatype);
+  for (const auto &field : cloud.fields)
+    fsize += field.count * pcl::getFieldSize (field.datatype);
 
   // Parse the cloud data and store it in the array
   double xy[2];
-  for (unsigned int d = 0; d < cloud.fields[field_idx].count; ++d)
+  for (uindex_t d = 0; d < cloud.fields[field_idx].count; ++d)
   {
     xy[0] = d;
     float data;
@@ -584,11 +477,7 @@ pcl::visualization::PCLHistogramVisualizer::ExitMainLoopTimerCallback::Execute (
     return;
 
   // Stop vtk loop and send notification to app to wake it up
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION <= 4))
-  interact->stopLoop ();
-#else
   interact->TerminateApp ();
-#endif
 }
 
 //////////////////////////////////////////////////////////////////////////////
